﻿using System;
using System.Collections;
using System.Collections.Concurrent;
using System.Collections.Generic;
using System.Runtime.InteropServices;

namespace Nerv35
{
    public interface IProxy
    {
        void on_connect();
        void on_disconnect();
    }
    public class Proxy
    {
        // nerv_proxy delegate
        public delegate void on_connect_ft(IntPtr _object_);
        public delegate void on_disconnect_ft(IntPtr _object_);

        static on_connect_ft on_connect_ = new on_connect_ft(on_connect);
        static on_disconnect_ft on_disconnect_ = new on_disconnect_ft(on_disconnect);

        [DllImport("64nerv35.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "nerv_proxy_alloc")]
        public static extern IntPtr alloc(on_connect_ft _on_connect_, on_disconnect_ft _on_disconnect_);

        [DllImport("64nerv35.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "nerv_proxy_free")]
        public static unsafe extern void free(IntPtr _object_);

        [DllImport("64nerv35.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "nerv_proxy_start")]
        public static extern Int32 start(IntPtr _object_, Address _stub_addr);

        [DllImport("64nerv35.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "nerv_proxy_stop")]
        public static extern Int32 stop(IntPtr _object_);

        [DllImport("64nerv35.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "nerv_proxy_is_connected")]
        public static extern Boolean is_connected(IntPtr _object_);

        [DllImport("64nerv35.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "nerv_proxy_is_started")]
        public static extern Boolean is_started(IntPtr _object_);

        [DllImport("64nerv35.dll", CallingConvention = CallingConvention.Cdecl, EntryPoint = "nerv_proxy_get_address")]
        public static extern Int32 get_address(IntPtr _object_, ref Address _proxy_addr_, ref Address _stub_addr_);

        static ConcurrentDictionary<IntPtr, Proxy> dict = new ConcurrentDictionary<IntPtr, Proxy>();

        public IntPtr object_;
        public IProxy iproxy;

        public static void on_connect(IntPtr _object_)
        {
            // nerv_stub object = ;
            dict[_object_].iproxy.on_connect();
        }

        public static void on_disconnect(IntPtr _object_)
        {
            dict[_object_].iproxy.on_disconnect();
        }

        public Proxy(IProxy _iproxy)
        {
            object_ = alloc(on_connect_, on_disconnect_);
            dict[object_] = this;
            iproxy = _iproxy;
        }

        ~Proxy()
        {
            // API.nerv_stub_free(inst_);
            // API.nerv_finalize()에서 정리되지 않은 객체를 자동해제하므로
            // C#에서는 API.nerv_stub_free(inst_); 로 정리하지 않는다.
            // C#의 소멸자는 언제 호출될지 알수 없다.

            Proxy obj;
            dict.TryRemove(object_, out obj);
        }

        public Int32 start(Address _stub_addr)
        {
            return start(object_, _stub_addr);
        }

        public Int32 stop()
        {
            return stop(object_);
        }

        public Boolean is_connected()
        {
            return is_connected(object_);
        }

        public Boolean is_started()
        {
            return is_started(object_);
        }

        public Int32 get_address(ref Address _proxy_addr_, ref Address _stub_addr_)
        {
            return get_address(object_, ref _proxy_addr_, ref _stub_addr_);
        }
    }
}
