<?xml version="1.0" encoding="UTF-8"?>
<!-- copyright@ 2020, Waregram , All right Reserved. -->
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:include href="../../../xsl_util.xsl"/>

<xsl:output method="text" encoding="utf-8" indent="yes"/>
<xsl:param name = "export_method_domain_path" />
<xsl:param name = "export_method_name" />
<xsl:template match="/">
  <xsl:for-each select="nerv_icde/methoddomain/row">
    <xsl:if test="path=$export_method_domain_path">
      <xsl:variable name="open_namespace_path">
        <xsl:call-template name="replace">
          <xsl:with-param name="text" select="path" />
          <xsl:with-param name="replace" select="'/'" />
          <xsl:with-param name="by" select="' { namespace '" />
        </xsl:call-template></xsl:variable>
      <xsl:variable name="close_namespace_path">
        <xsl:call-template name="replace">
          <xsl:with-param name="text" select="path" />
          <xsl:with-param name="replace" select="'/'" />
          <xsl:with-param name="by" select="' */ } /* '" />
        </xsl:call-template>
      </xsl:variable>
      <xsl:variable name="define_path">
        <xsl:call-template name="replace">
          <xsl:with-param name="text" select="path" />
          <xsl:with-param name="replace" select="'/'" />
          <xsl:with-param name="by" select="'__'" />
        </xsl:call-template>
      </xsl:variable>
/*******************************************************************************
    Document Header
    domain   : <xsl:value-of select="name"/>
    creator  : <xsl:value-of select="author"/>
    create date : <xsl:value-of select="create-day"/>
    description : <xsl:value-of select="description"/>
*******************************************************************************/
      <xsl:for-each select="method/row">
        <xsl:if test="name=$export_method_name">
          <xsl:variable name="in_data_domain_namespace">
          <xsl:call-template name="get_data_domain_namespace_cs">
            <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
          </xsl:call-template>
          </xsl:variable>
          <xsl:variable name="out_data_domain_namespace">
          <xsl:call-template name="get_data_domain_namespace_cs">
            <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
          </xsl:call-template>
          </xsl:variable>
/*******************************************************************************
    Method       : <xsl:value-of select="name"/>
    Input        : <xsl:if test="in_data != '(null)'"><xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/></xsl:if>
    output       : <xsl:if test="out_data != '(null)'"><xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/></xsl:if>
    Method Type  : <xsl:value-of select="type"/>
    Code         : <xsl:value-of select="code"/>
    Writer       : <xsl:value-of select="../../author"/>
    Description  : <xsl:value-of select="../../description"/>
*******************************************************************************/
using System;
using System.Runtime.InteropServices;
using System.Collections.Concurrent;
using Nerv35;

namespace Nerv35 { namespace Method <xsl:value-of select="$open_namespace_path" /> {

          <xsl:if test="type='CALL'" ><xsl:call-template name="template_call" /></xsl:if>
          <xsl:if test="type='COMMAND'" ><xsl:call-template name="template_command" /></xsl:if>
          <xsl:if test="type='DATAGRAM'" ><xsl:call-template name="template_datagram" /></xsl:if>
          <xsl:if test="type='INFORMATION'" ><xsl:call-template name="template_information" /></xsl:if>
          <xsl:if test="type='EVENT'" ><xsl:call-template name="template_event" /></xsl:if>
          <xsl:if test="type='SIGNAL'" ><xsl:call-template name="template_signal" /></xsl:if>

} } /* <xsl:value-of select="$close_namespace_path" /> */

        </xsl:if>
      </xsl:for-each>
    </xsl:if>
  </xsl:for-each>
</xsl:template>

<xsl:template name ="template_datagram" >
  <xsl:variable name="in_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace_cs">
      <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="out_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace_cs">
      <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>

// proxy
public interface IProxy_method_<xsl:value-of select="name"/>
{
    void on_datagram( Address _stub_addr,<xsl:if test="out_data!='(null)'"> ref Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.<xsl:value-of select="out_data"/> _dtg_data_,</xsl:if> Double _elapsed_time);
}
public unsafe class Proxy_method_<xsl:value-of select="name"/>
{
    static Nerv35.Proxy_datagram.on_datagram_ft on_datagram_     = new Nerv35.Proxy_datagram.on_datagram_ft(on_datagram);

    static ConcurrentDictionary&lt;IntPtr, Proxy_method_<xsl:value-of select="name"/>&gt; dict = new ConcurrentDictionary&lt;IntPtr, Proxy_method_<xsl:value-of select="name"/>&gt;();

    public static void on_datagram(IntPtr _datagram_, Address _stub_addr, IntPtr _dtg_data_, UInt32 _dtg_size, Double _elapsed_time)
    {<xsl:if test="out_data!='(null)'">
        Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.<xsl:value-of select="out_data"/>* dtg_data_ = (Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.<xsl:value-of select="out_data"/>*)_dtg_data_;
        </xsl:if>
        dict[_datagram_].idatagram.on_datagram(_stub_addr, <xsl:if test="out_data!='(null)'">ref *dtg_data_, </xsl:if>_elapsed_time);
    }

    public IntPtr datagram_;
    public IProxy_method_<xsl:value-of select="name"/> idatagram;

    public Proxy_method_<xsl:value-of select="name"/>(ref Nerv35.Proxy _object_, IProxy_method_<xsl:value-of select="name"/> _idatagram)
    {
        datagram_ = Nerv35.Proxy_datagram.alloc(<xsl:value-of select="code"/>, _object_.object_,on_datagram);
        dict[datagram_] = this;
        idatagram = _idatagram;
    }
    ~Proxy_method_<xsl:value-of select="name"/>()
    {
        // Nerv35.Proxy_datagram.free(datagram_);
        // API.nerv_finalize()에서 정리되지 않은 객체를 자동해제하므로
        // C#에서는 Nerv35.Proxy_datagram.free(datagram_); 로 정리하지 않는다.
        // C#의 소멸자는 언제 호출될지 알수 없다.

        Proxy_method_<xsl:value-of select="name"/> datagram_reference;
        dict.TryRemove(datagram_, out datagram_reference);
    }
}

// stub
public interface IStub_method_<xsl:value-of select="name"/>
{
}
public unsafe class Stub_method_<xsl:value-of select="name"/>
{
    static ConcurrentDictionary&lt;IntPtr, Stub_method_<xsl:value-of select="name"/>&gt; dict = new ConcurrentDictionary&lt;IntPtr, Stub_method_<xsl:value-of select="name"/>&gt;();

    public IntPtr datagram_;
    public IStub_method_<xsl:value-of select="name"/> idatagram;

    public Stub_method_<xsl:value-of select="name"/>(ref Nerv35.Stub _object_, IStub_method_<xsl:value-of select="name"/> _idatagram)
    {
        datagram_ = Nerv35.Stub_datagram.alloc(<xsl:value-of select="code"/>, _object_.object_);
        dict[datagram_] = this;
        idatagram = _idatagram;
    }
    ~Stub_method_<xsl:value-of select="name"/>()
    {
        // Nerv35.Stub_datagram.free(datagram_);
        // API.nerv_finalize()에서 정리되지 않은 객체를 자동해제하므로
        // C#에서는 Nerv35.Stub_datagram.free(datagram_); 로 정리하지 않는다.
        // C#의 소멸자는 언제 호출될지 알수 없다.

        Stub_method_<xsl:value-of select="name"/> datagram_reference;
        dict.TryRemove(datagram_, out datagram_reference);
    }

    public Int32 cast(UInt32 _ip<xsl:if test="out_data!='(null)'">, ref Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.<xsl:value-of select="out_data"/> _dtg_data_</xsl:if>)
    {
  <xsl:if test="out_data!='(null)'">
        fixed (Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.<xsl:value-of select="out_data"/>* dtg_data_ = &amp;_dtg_data_)
        {
            Int32 err = Nerv35.Stub_datagram.cast(datagram_, _ip, (IntPtr)dtg_data_, _dtg_data_.fit_size() );
            return err;
        }
  </xsl:if>
  <xsl:if test="out_data='(null)'">
        Int32 err = Nerv35.Stub_datagram.cast(datagram_, _ip, IntPtr.Zero, 0);
        return err;
  </xsl:if>
    }


    public Int32 broadcast(<xsl:if test="out_data!='(null)'">ref Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.<xsl:value-of select="out_data"/> _dtg_data_</xsl:if>)
    {
  <xsl:if test="out_data!='(null)'">
        fixed (Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.<xsl:value-of select="out_data"/>* dtg_data_ = &amp;_dtg_data_)
        {
            Int32 err = Nerv35.Stub_datagram.broadcast(datagram_, (IntPtr)dtg_data_, _dtg_data_.fit_size() );
            return err;
        }
  </xsl:if>
  <xsl:if test="out_data='(null)'">
        Int32 err = Nerv35.Stub_datagram.broadcast(datagram_, IntPtr.Zero, 0);
        return err;
  </xsl:if>
    }

}

</xsl:template>


<xsl:template name ="template_command" >
  <xsl:variable name="in_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace_cs">
      <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="out_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace_cs">
      <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
// proxy
public interface IProxy_method_<xsl:value-of select="name"/>
{
    void on_unlink(Int32 _err);
}
public unsafe class Proxy_method_<xsl:value-of select="name"/>
{
    static Nerv35.Proxy_command.on_unlink_ft on_unlink_ = new Nerv35.Proxy_command.on_unlink_ft(on_unlink);
    static ConcurrentDictionary&lt;IntPtr, Proxy_method_<xsl:value-of select="name"/>&gt; dict = new ConcurrentDictionary&lt;IntPtr, Proxy_method_<xsl:value-of select="name"/>&gt;();

    public static void on_unlink(IntPtr _command_, Int32 _error)
    {
        dict[_command_].icommand.on_unlink(_error);
    }

    public IntPtr command_;
    public IProxy_method_<xsl:value-of select="name"/> icommand;

    public Proxy_method_<xsl:value-of select="name"/>(ref Nerv35.Proxy _object_, IProxy_method_<xsl:value-of select="name"/> _icommand)
    {
        command_ = Nerv35.Proxy_command.alloc(<xsl:value-of select="code"/>, _object_.object_,on_unlink_);
        dict[command_] = this;
        icommand = _icommand;
    }
    ~Proxy_method_<xsl:value-of select="name"/>()
    {
        // Nerv35.Proxy_command.free(command_);
        // API.nerv_finalize()에서 정리되지 않은 객체를 자동해제하므로
        // C#에서는 Nerv35.Proxy_command.free(command_); 로 정리하지 않는다.
        // C#의 소멸자는 언제 호출될지 알수 없다.

        Proxy_method_<xsl:value-of select="name"/> command_reference;
        dict.TryRemove(command_, out command_reference);
    }

    public Boolean is_linked()
    {
        Boolean b = Nerv35.Proxy_command.is_linked(command_);
        return b;
    }

    public IntPtr async_link(Double _limit_time)
    {
        IntPtr promise_ = Nerv35.Proxy_command.async_link(command_, _limit_time);
        return promise_;
    }

    public Int32 async_link_return(IntPtr _promise_, ref Double _time_)
    {
        Int32 err = Nerv35.Proxy_command.async_link_return(command_, _promise_, ref _time_);
        return err;
    }

    public Int32 link(Double _limit_time, ref Double _time_)
    {
        IntPtr promise_ = Nerv35.Proxy_command.async_link( command_, _limit_time);
        Int32 err = Nerv35.Proxy_command.async_link_return(command_, promise_, ref _time_);
        return err;
    }

    public void unlink()
    {
        Nerv35.Proxy_command.unlink(command_);
    }

    public void set_sync_model(Boolean _reliability, Sync_model _sync_model, Double _interval)
    {
        Nerv35.Proxy_command.set_sync_model(command_, _reliability, _sync_model, _interval);
    }

    public IntPtr see_history(UIntPtr _history_size)
    {
        IntPtr letter_ = Nerv35.Proxy_command.see_history(command_, _history_size);
        return letter_;
    }

    public void set_history_size(UIntPtr _history_size)
    {
        Nerv35.Proxy_command.set_history_size(command_, _history_size);
    }
  <xsl:if test="in_data!='(null)'">
    public ref Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.<xsl:value-of select="in_data"/> get()
    {
        IntPtr cmd_data_ = Nerv35.Proxy_command.get(command_, Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.<xsl:value-of select="in_data"/>._size);
        return ref *(Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.<xsl:value-of select="in_data"/>*)cmd_data_;
    }

    public void cancel()
    {
        Nerv35.Proxy_command.cancel(command_);
    }

    public Int32 post(ref Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.<xsl:value-of select="in_data"/> _cmd_data_ )
    {
        // 데이터를 post함수를 통해서 전달하지는 않고, 다만 크기만 지정한다
        // post 데이터 메모리는 get 으로 획득한 공간이다.
        Int32 err = Nerv35.Proxy_command.post(command_, _cmd_data_.fit_size() );
        return err;
    }

    public Int32 read(ref Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.<xsl:value-of select="in_data"/> _cmd_data_)
    {
        UInt32 cmd_size = Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.<xsl:value-of select="in_data"/>._size;

        fixed ( Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.<xsl:value-of select="in_data"/>* cmd_data_ = &amp;_cmd_data_)
        {
            Int32 err = Nerv35.Proxy_command.read(command_, (IntPtr)cmd_data_, ref cmd_size);
            return err;
        }
    }
  </xsl:if>

    public Int32 write(<xsl:if test="in_data!='(null)'">ref Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.<xsl:value-of select="in_data"/> _cmd_data_</xsl:if>)
    {
  <xsl:if test="in_data!='(null)'">
        fixed (Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.<xsl:value-of select="in_data"/>* cmd_data_ = &amp;_cmd_data_)
        {
            Int32 err = Nerv35.Proxy_command.write(command_, (IntPtr)cmd_data_, _cmd_data_.fit_size() );
            return err;
        }
  </xsl:if>
  <xsl:if test="in_data='(null)'">
        Int32 err = Nerv35.Proxy_command.write(command_, IntPtr.Zero, 0);
        return err;
  </xsl:if>
    }
}

// stub
public interface IStub_method_<xsl:value-of select="name"/>
{
    Int32 on_link(Address _proxy_addr, Double _packet_live_time, Double _packet_limit_time);
    void on_unlink(Address _proxy_addr, Int32 _err);
    void on_sync(Address _proxy_addr, Sync_model _sync_model, Double _elapsed_time, Double _limit<xsl:if test="in_data!='(null)'">, ref Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.<xsl:value-of select="in_data"/> _cmd_data_</xsl:if>);
}

public unsafe class Stub_method_<xsl:value-of select="name"/>
{
    static Nerv35.Stub_command.on_link_ft on_link_     = new Nerv35.Stub_command.on_link_ft(on_link);
    static Nerv35.Stub_command.on_unlink_ft on_unlink_ = new Nerv35.Stub_command.on_unlink_ft(on_unlink);
    static Nerv35.Stub_command.on_sync_ft on_sync_     = new Nerv35.Stub_command.on_sync_ft(on_sync);

    static ConcurrentDictionary&lt;IntPtr, Stub_method_<xsl:value-of select="name"/>&gt; dict = new ConcurrentDictionary&lt;IntPtr, Stub_method_<xsl:value-of select="name"/>&gt;();

    // nerv_stub_command delegate
    public static Int32 on_link(IntPtr _command_, Address _proxy_addr, Double _packet_live_time, Double _packet_limit_time)
    {
        Int32 err = dict[_command_].icommand.on_link(_proxy_addr, _packet_live_time, _packet_limit_time);
        return err;
    }
    public static void on_unlink(IntPtr _command_, Address _proxy_addr, Int32 _err)
    {
        dict[_command_].icommand.on_unlink(_proxy_addr, _err);
    }
    public static void on_sync(IntPtr _command_, Address _proxy_addr, Sync_model _sync_model, Double _elapsed_time, Double _limit, IntPtr _cmd_data_, UInt32 _cmd_size)
    {<xsl:if test="in_data!='(null)'">
        Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.<xsl:value-of select="in_data"/>* cmd_data_ = (Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.<xsl:value-of select="in_data"/>*)_cmd_data_;
        </xsl:if>
        dict[_command_].icommand.on_sync(_proxy_addr, _sync_model, _elapsed_time, _limit<xsl:if test="in_data!='(null)'">, ref *cmd_data_</xsl:if>);
    }

    public IntPtr command_;
    public IStub_method_<xsl:value-of select="name"/> icommand;

    public Stub_method_<xsl:value-of select="name"/>(ref Nerv35.Stub _object_, IStub_method_<xsl:value-of select="name"/> _icommand)
    {
        command_ = Nerv35.Stub_command.alloc(<xsl:value-of select="code"/>, _object_.object_, on_sync_, on_link_, on_unlink_);
        dict[command_] = this;
        icommand = _icommand;
    }
    ~Stub_method_<xsl:value-of select="name"/>()
    {
        // Nerv35.Stub_command.free(command_);
        // API.nerv_finalize()에서 정리되지 않은 객체를 자동해제하므로
        // C#에서는 Nerv35.Stub_command.free(command_); 로 정리하지 않는다.
        // C#의 소멸자는 언제 호출될지 알수 없다.

        Stub_method_<xsl:value-of select="name"/> command_reference;
        dict.TryRemove(command_, out command_reference);
    }

    public void unlink(Address _proxy_addr)
    {
        Nerv35.Stub_command.unlink(command_, _proxy_addr);
    }

    public void unlink_all_proxy()
    {
        Nerv35.Stub_command.unlink_all_proxy(command_);
    }

    public void set_sync(Boolean _use_on_sync)
    {
        Nerv35.Stub_command.set_sync(command_, _use_on_sync);
    }

    public IntPtr see_history(Address _proxy_addr, UIntPtr _history_size)
    {
        IntPtr letter_ = Nerv35.Stub_command.see_history(command_, _proxy_addr, _history_size);
        return letter_;
    }

    public void set_history_size(UIntPtr _history_size)
    {
        Nerv35.Stub_command.set_history_size(command_, _history_size);
    }
  <xsl:if test="in_data!='(null)'">
    public Int32 read(Address _proxy_addr, ref Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.<xsl:value-of select="in_data"/> _cmd_data_)
    {
        UInt32 cmd_size = Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.<xsl:value-of select="in_data"/>._size;

        fixed ( Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.<xsl:value-of select="in_data"/>* cmd_data_ = &amp;_cmd_data_)
        {
            Int32 err = Nerv35.Stub_command.read(command_, _proxy_addr, (IntPtr)cmd_data_, ref cmd_size);
            return err;
        }
    }
  </xsl:if>
}
</xsl:template>


<xsl:template name ="template_information" >
  <xsl:variable name="in_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace_cs">
      <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="out_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace_cs">
      <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
// proxy
public interface IProxy_method_<xsl:value-of select="name"/>
{
    void on_sync(Sync_model _sync_model, Double _elapsed_time, Double _limit<xsl:if test="out_data!='(null)'">, ref Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.<xsl:value-of select="out_data"/> _inf_data_</xsl:if>);
    void on_unlink(Int32 _err);
}
public unsafe class Proxy_method_<xsl:value-of select="name"/>
{
    static Nerv35.Proxy_information.on_unlink_ft on_unlink_ = new Nerv35.Proxy_information.on_unlink_ft(on_unlink);
    static Nerv35.Proxy_information.on_sync_ft on_sync_     = new Nerv35.Proxy_information.on_sync_ft(on_sync);

    static ConcurrentDictionary&lt;IntPtr, Proxy_method_<xsl:value-of select="name"/>&gt; dict = new ConcurrentDictionary&lt;IntPtr, Proxy_method_<xsl:value-of select="name"/>&gt;();

    public static void on_sync(IntPtr _information_, Sync_model _sync_model, Double _elapsed_time, Double _limit, IntPtr _inf_data_, UInt32 _inf_size)
    {<xsl:if test="out_data!='(null)'">
        Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.<xsl:value-of select="out_data"/>* inf_data_ = (Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.<xsl:value-of select="out_data"/>*)_inf_data_;
        </xsl:if>
        dict[_information_].iinformation.on_sync(_sync_model, _elapsed_time, _limit<xsl:if test="out_data!='(null)'">, ref *inf_data_</xsl:if>);
    }
    public static void on_unlink(IntPtr _information_, Int32 _error)
    {
        dict[_information_].iinformation.on_unlink(_error);
    }

    public IntPtr information_;
    public IProxy_method_<xsl:value-of select="name"/> iinformation;

    public Proxy_method_<xsl:value-of select="name"/>(ref Nerv35.Proxy _object_, IProxy_method_<xsl:value-of select="name"/> _iinformation)
    {
        information_ = Nerv35.Proxy_information.alloc(<xsl:value-of select="code"/>, _object_.object_,on_unlink_,on_sync_);
        dict[information_] = this;
        iinformation = _iinformation;
    }
    ~Proxy_method_<xsl:value-of select="name"/>()
    {
        // Nerv35.Proxy_information.free(information_);
        // API.nerv_finalize()에서 정리되지 않은 객체를 자동해제하므로
        // C#에서는 Nerv35.Proxy_information.free(information_); 로 정리하지 않는다.
        // C#의 소멸자는 언제 호출될지 알수 없다.

        Proxy_method_<xsl:value-of select="name"/> information_reference;
        dict.TryRemove(information_, out information_reference);
    }

    public Boolean is_linked()
    {
        Boolean b = Nerv35.Proxy_information.is_linked(information_);
        return b;
    }

    public IntPtr async_link(Boolean _reliability, Double _limit_time)
    {
        IntPtr promise_ = Nerv35.Proxy_information.async_link(information_, _reliability, _limit_time);
        return promise_;
    }

    public Int32 async_link_return(IntPtr _promise_, ref Double _time_)
    {
        Int32 err = Nerv35.Proxy_information.async_link_return(information_, _promise_, ref _time_);
        return err;
    }

    public Int32 link(Boolean _reliability, Double _limit_time, ref Double _time_)
    {
        IntPtr promise_ = Nerv35.Proxy_information.async_link( information_, _reliability, _limit_time);
        Int32 err = Nerv35.Proxy_information.async_link_return(information_, promise_, ref _time_);
        return err;
    }

    public void unlink()
    {
        Nerv35.Proxy_information.unlink(information_);
    }

    public void set_sync(Boolean _use_on_sync)
    {
        Nerv35.Proxy_information.set_sync(information_, _use_on_sync);
    }

    public IntPtr see_history(UIntPtr _size)
    {
        IntPtr letter_ = Nerv35.Proxy_information.see_history(information_, _size);
        return letter_;
    }

    public void set_history_size(UIntPtr _history_size)
    {
        Nerv35.Proxy_information.set_history_size(information_, _history_size);
    }

  <xsl:if test="out_data!='(null)'">
    public Int32 read(ref Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.<xsl:value-of select="out_data"/> _inf_data_)
    {
        UInt32 inf_size = Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.<xsl:value-of select="out_data"/>._size;

        fixed ( Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.<xsl:value-of select="out_data"/>* inf_data_ = &amp;_inf_data_)
        {
            Int32 err = Nerv35.Proxy_information.read(information_, (IntPtr)inf_data_, ref inf_size);
            return err;
        }
    }
  </xsl:if>
}

// stub
public interface IStub_method_<xsl:value-of select="name"/>
{
    Int32 on_link(Address _proxy_addr, Boolean _reliability, Double _packet_live_time, Double _packet_limit_time);
    void on_unlink(Address _proxy_addr, Int32 _err);
}

public unsafe class Stub_method_<xsl:value-of select="name"/>
{
    static Nerv35.Stub_information.on_link_ft on_link_     = new Nerv35.Stub_information.on_link_ft(on_link);
    static Nerv35.Stub_information.on_unlink_ft on_unlink_ = new Nerv35.Stub_information.on_unlink_ft(on_unlink);
    static ConcurrentDictionary&lt;IntPtr, Stub_method_<xsl:value-of select="name"/>&gt; dict = new ConcurrentDictionary&lt;IntPtr, Stub_method_<xsl:value-of select="name"/>&gt;();

    // nerv_stub_information delegate
    public static Int32 on_link(IntPtr _information_, Address _proxy_addr, Boolean _reliability, Double _packet_live_time, Double _packet_limit_time)
    {
        Int32 err = dict[_information_].iinformation.on_link(_proxy_addr, _reliability, _packet_live_time, _packet_limit_time);
        return err;
    }
    public static void on_unlink(IntPtr _information_, Address _proxy_addr, Int32 _err)
    {
        dict[_information_].iinformation.on_unlink(_proxy_addr, _err);
    }

    public IntPtr information_;
    public IStub_method_<xsl:value-of select="name"/> iinformation;

    public Stub_method_<xsl:value-of select="name"/>(ref Nerv35.Stub _object_, IStub_method_<xsl:value-of select="name"/> _iinformation)
    {
        information_ = Nerv35.Stub_information.alloc(<xsl:value-of select="code"/>, _object_.object_, on_link_, on_unlink_);
        dict[information_] = this;
        iinformation = _iinformation;
    }
    ~Stub_method_<xsl:value-of select="name"/>()
    {
        // Nerv35.Stub_information.free(information_);
        // API.nerv_finalize()에서 정리되지 않은 객체를 자동해제하므로
        // C#에서는 Nerv35.Stub_information.free(information_); 로 정리하지 않는다.
        // C#의 소멸자는 언제 호출될지 알수 없다.

        Stub_method_<xsl:value-of select="name"/> information_reference;
        dict.TryRemove(information_, out information_reference);
    }

    public void unlink(Address _proxy_addr)
    {
        Nerv35.Stub_information.unlink(information_, _proxy_addr);
    }

    public void unlink_all_proxy()
    {
        Nerv35.Stub_information.unlink_all_proxy(information_);
    }

  <xsl:if test="out_data!='(null)'">
    public ref Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.<xsl:value-of select="out_data"/> get()
    {
        IntPtr inf_data_ = Nerv35.Stub_information.get(information_, Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.<xsl:value-of select="out_data"/>._size);
        return ref *(Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.<xsl:value-of select="out_data"/>*)inf_data_;
    }

    public void cancel()
    {
        Nerv35.Stub_information.cancel(information_);
    }

    public Int32 post(ref Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.<xsl:value-of select="out_data"/> _inf_data_ )
    {
        // 데이터를 post함수를 통해서 전달하지는 않고, 다만 크기만 지정한다
        // post 데이터 메모리는 get 으로 획득한 공간이다.
        Int32 err = Nerv35.Stub_information.post(information_, _inf_data_.fit_size() );
        return err;
    }

    public Int32 write(ref Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.<xsl:value-of select="out_data"/> _inf_data_)
    {
        fixed (Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.<xsl:value-of select="out_data"/>* inf_data_ = &amp;_inf_data_)
        {
            Int32 err = Nerv35.Stub_information.write(information_, (IntPtr)inf_data_, _inf_data_.fit_size() );
            return err;
        }
    }

    public Int32 read(ref Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.<xsl:value-of select="out_data"/> _inf_data_)
    {
        UInt32 inf_size = Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.<xsl:value-of select="out_data"/>._size;

        fixed ( Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.<xsl:value-of select="out_data"/>* inf_data_ = &amp;_inf_data_)
        {
            Int32 err = Nerv35.Proxy_information.read(information_, (IntPtr)inf_data_, ref inf_size);
            return err;
        }
    }
  </xsl:if>
    public IntPtr see_history(UIntPtr _size)
    {
        IntPtr letter_ = Nerv35.Stub_information.see_history(information_, _size);
        return letter_;
    }

    public void set_history_size(UIntPtr _history_size)
    {
        Nerv35.Stub_information.set_history_size(information_, _history_size);
    }
}
</xsl:template>



<xsl:template name ="template_call" >
  <xsl:variable name="in_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace_cs">
      <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="out_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace_cs">
      <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
// proxy
public interface IProxy_method_<xsl:value-of select="name"/>
{
}
public unsafe class Proxy_method_<xsl:value-of select="name"/>
{
    static ConcurrentDictionary&lt;IntPtr, Proxy_method_<xsl:value-of select="name"/>&gt; dict = new ConcurrentDictionary&lt;IntPtr, Proxy_method_<xsl:value-of select="name"/>&gt;();

    public IntPtr call_;
    public IProxy_method_<xsl:value-of select="name"/> icall;

    public Proxy_method_<xsl:value-of select="name"/>(ref Nerv35.Proxy _object_, IProxy_method_<xsl:value-of select="name"/> _icall)
    {
        call_ = Nerv35.Proxy_call.alloc(<xsl:value-of select="code"/>, _object_.object_);
        dict[call_] = this;
        icall = _icall;
    }
    ~Proxy_method_<xsl:value-of select="name"/>()
    {
        // Nerv35.Proxy_call.free(call_);
        // API.nerv_finalize()에서 정리되지 않은 객체를 자동해제하므로
        // C#에서는 Nerv35.Proxy_call.free(call_); 로 정리하지 않는다.
        // C#의 소멸자는 언제 호출될지 알수 없다.

        Proxy_method_<xsl:value-of select="name"/> call;
        dict.TryRemove(call_, out call);
    }


    public IntPtr async_call(<xsl:if test="in_data!='(null)'">ref Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.<xsl:value-of select="in_data"/> _cal_data_,</xsl:if> Double _limit_time)
    {
        <xsl:if test="in_data!='(null)'">
        IntPtr promise_;
        UInt32 cal_size = (UInt32)_cal_data_.fit_size();
        fixed ( Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.<xsl:value-of select="in_data"/>* cal_data_ = &amp;_cal_data_)
        {
            promise_ = Nerv35.Proxy_call.async_call( call_, (IntPtr)cal_data_, cal_size, _limit_time);
        }
        </xsl:if>
        <xsl:if test="in_data='(null)'">
        IntPtr promise_ = Nerv35.Proxy_call.async_call( call_, IntPtr.Zero, 0, _limit_time);
        </xsl:if>
        return promise_;
    }


    public Int32 async_return(IntPtr _promise_, <xsl:if test="out_data!='(null)'">ref Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.<xsl:value-of select="out_data"/> _return_data_,</xsl:if> ref Double _time_ )
    {
        <xsl:if test="out_data!='(null)'">
        UInt32 return_size = (UInt32)_return_data_._size;
        fixed ( Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.<xsl:value-of select="out_data"/>* return_data_ = &amp;_return_data_)
        {
            Int32 err = Nerv35.Proxy_call.async_return(call_, _promise_, (IntPtr)return_data_, ref return_size, ref _time_);
            return err;
        }
        </xsl:if>
        <xsl:if test="out_data='(null)'">
        UInt32 return_size = 0;
        Int32 err = Nerv35.Proxy_call.async_return(call_, _promise_, IntPtr.Zero, ref return_size, ref _time_);
        return err;
        </xsl:if>
    }

    public Int32 call(<xsl:if test="in_data!='(null)'">ref Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.<xsl:value-of select="in_data"/> _cal_data_,</xsl:if>
                      <xsl:if test="out_data!='(null)'">ref Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.<xsl:value-of select="out_data"/> _return_data_,</xsl:if>
                      Double _limit_time , ref Double _time_ )
    {
        <xsl:if test="in_data!='(null)'">
        IntPtr promise_;
        UInt32 cal_size = (UInt32)_cal_data_.fit_size();
        fixed ( Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.<xsl:value-of select="in_data"/>* cal_data_ = &amp;_cal_data_)
        {
            promise_ = Nerv35.Proxy_call.async_call( call_, (IntPtr)cal_data_, cal_size, _limit_time);
        }
        </xsl:if>
        <xsl:if test="in_data='(null)'">
        promise_ = Nerv35.Proxy_call.async_call( call_, IntPtr.Zero, 0, _limit_time);
        </xsl:if>
        <xsl:if test="out_data!='(null)'">
        UInt32 return_size = (UInt32)_return_data_._size;
        fixed ( Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.<xsl:value-of select="out_data"/>* return_data_ = &amp;_return_data_)
        {
            Int32 err = Nerv35.Proxy_call.async_return(call_, promise_, (IntPtr)return_data_, ref return_size, ref _time_);
            return err;
        }
        </xsl:if>
        <xsl:if test="out_data='(null)'">
        UInt32 return_size = 0;
        Int32 err = Nerv35.Proxy_call.async_return(call_, promise_, IntPtr.Zero, ref return_size, ref _time_);
        return err;
        </xsl:if>

    }

    public void cancel(Int32 _error)
    {
        Nerv35.Proxy_call.cancel(call_, _error);
    }
}

// stub
public interface IStub_method_<xsl:value-of select="name"/>
{
    Int32 on_call( Address _proxy_addr, <xsl:if test="in_data!='(null)'">ref Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.<xsl:value-of select="in_data"/> _cal_data_,</xsl:if> <xsl:if test="out_data!='(null)'">ref Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.<xsl:value-of select="out_data"/> _return_data_,</xsl:if> double _elapsed_time, double _packet_limit_time);
    void on_cancel(Address _proxy_addr, Int32 _error);
}
public unsafe class Stub_method_<xsl:value-of select="name"/>
{
    static Nerv35.Stub_call.on_call_ft on_call_ = new Nerv35.Stub_call.on_call_ft(on_call);
    static Nerv35.Stub_call.on_cancel_ft on_cancel_ = new Nerv35.Stub_call.on_cancel_ft(on_cancel);
    static ConcurrentDictionary&lt;IntPtr, Stub_method_<xsl:value-of select="name"/>&gt; dict = new ConcurrentDictionary&lt;IntPtr, Stub_method_<xsl:value-of select="name"/>&gt;();

    public static Int32 on_call(IntPtr _call_,  Address _proxy_addr, IntPtr _cal_data_, UInt32 _cal_size, IntPtr _return_data_, ref UInt32 _return_size,  double _elapsed_time, double _packet_limit_time)
    {<xsl:if test="in_data!='(null)'">
        Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.<xsl:value-of select="in_data"/>* cal_data_ = (Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.<xsl:value-of select="in_data"/>*)_cal_data_;
        </xsl:if>
        <xsl:if test="out_data!='(null)'">
        Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.<xsl:value-of select="in_data"/>* return_data_ = (Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.<xsl:value-of select="in_data"/>*)_return_data_;
        </xsl:if>
        return dict[_call_].icall.on_call(_proxy_addr,<xsl:if test="in_data!='(null)'"> ref *cal_data_,</xsl:if><xsl:if test="out_data!='(null)'"> ref *return_data_,</xsl:if> _elapsed_time, _packet_limit_time);
    }
    public static void on_cancel(IntPtr _call_, Address _proxy_addr, Int32 _error)
    {
        dict[_call_].icall.on_cancel(_proxy_addr, _error);
    }

    public IntPtr call_;
    public IStub_method_<xsl:value-of select="name"/> icall;

    public Stub_method_<xsl:value-of select="name"/>(ref Nerv35.Stub _object_, IStub_method_<xsl:value-of select="name"/> _icall)
    {
        call_ = Nerv35.Stub_call.alloc(<xsl:value-of select="code"/>, <xsl:if test="out_data!='(null)'">Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.<xsl:value-of select="out_data"/>._size</xsl:if><xsl:if test="out_data='(null)'">0</xsl:if>, _object_.object_, on_call_, on_cancel_);
        dict[call_] = this;
        icall = _icall;
    }
    ~Stub_method_<xsl:value-of select="name"/>()
    {
        // Nerv35.Stub_call.free(call_);
        // API.nerv_finalize()에서 정리되지 않은 객체를 자동해제하므로
        // C#에서는 Nerv35.Stub_call.free(call_); 로 정리하지 않는다.
        // C#의 소멸자는 언제 호출될지 알수 없다.

        Stub_method_<xsl:value-of select="name"/> call;
        dict.TryRemove(call_, out call);
    }
}
</xsl:template>


<xsl:template name ="template_signal" >
  <xsl:variable name="in_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace_cs">
      <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="out_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace_cs">
      <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>


// proxy
public interface IProxy_method_<xsl:value-of select="name"/>
{
}
public unsafe class Proxy_method_<xsl:value-of select="name"/>
{
    static ConcurrentDictionary&lt;IntPtr, Proxy_method_<xsl:value-of select="name"/>&gt; dict = new ConcurrentDictionary&lt;IntPtr, Proxy_method_<xsl:value-of select="name"/>&gt;();

    public IntPtr signal_;
    public IProxy_method_<xsl:value-of select="name"/> isignal;

    public Proxy_method_<xsl:value-of select="name"/>(ref Nerv35.Proxy _object_, IProxy_method_<xsl:value-of select="name"/> _isignal)
    {
        signal_ = Nerv35.Proxy_signal.alloc(<xsl:value-of select="code"/>, _object_.object_);
        dict[signal_] = this;
        isignal = _isignal;
    }
    ~Proxy_method_<xsl:value-of select="name"/>()
    {
        // Nerv35.Proxy_signal.free(signal_);
        // API.nerv_finalize()에서 정리되지 않은 객체를 자동해제하므로
        // C#에서는 Nerv35.Proxy_signal.free(signal_); 로 정리하지 않는다.
        // C#의 소멸자는 언제 호출될지 알수 없다.

        Proxy_method_<xsl:value-of select="name"/> signal;
        dict.TryRemove(signal_, out signal);
    }


    public IntPtr async_signal(<xsl:if test="in_data!='(null)'">ref Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.<xsl:value-of select="in_data"/> _sig_data_,</xsl:if> Double _limit_time)
    {
        <xsl:if test="in_data!='(null)'">
        UInt32 sig_size = (UInt32)_sig_data_.fit_size();
        fixed ( Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.<xsl:value-of select="in_data"/>* sig_data_ = &amp;_sig_data_)
        {
            IntPtr promise_ = Nerv35.Proxy_signal.async_signal( signal_, (IntPtr)sig_data_, sig_size, _limit_time);
            return promise_;
        }
        </xsl:if>
        <xsl:if test="in_data='(null)'">
        IntPtr promise_ = Nerv35.Proxy_signal.async_signal( signal_, IntPtr.Zero, 0, _limit_time);
        return promise_;
        </xsl:if>
    }


    public Int32 async_return(IntPtr _promise_, ref Double _time_ )
    {
        Int32 err = Nerv35.Proxy_signal.async_return(signal_, _promise_, ref _time_);
        return err;
    }

    public Int32 signal(<xsl:if test="in_data!='(null)'">ref Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.<xsl:value-of select="in_data"/> _sig_data_,</xsl:if> Double _limit_time , ref Double _time_ )
    {
        <xsl:if test="in_data!='(null)'">
        UInt32 sig_size = (UInt32)_sig_data_.fit_size();
        fixed ( Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.<xsl:value-of select="in_data"/>* sig_data_ = &amp;_sig_data_)
        {
            Int32 err = Nerv35.Proxy_signal.signal( signal_, (IntPtr)sig_data_, sig_size, _limit_time, ref _time_);
            return err;
        }
        </xsl:if>
        <xsl:if test="in_data='(null)'">
        Int32 err = Nerv35.Proxy_signal.signal( signal_, IntPtr.Zero, 0, _limit_time, ref _time_);
        return err;
        </xsl:if>
    }

    public void cancel(Int32 _error)
    {
        Nerv35.Proxy_signal.cancel(signal_, _error);
    }
}

// stub
public interface IStub_method_<xsl:value-of select="name"/>
{
    void on_signal( Address _proxy_addr, <xsl:if test="in_data!='(null)'">ref Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.<xsl:value-of select="in_data"/> _sig_data_,</xsl:if> double _elapsed_time);
    void on_cancel(Address _proxy_addr, Int32 _error);
}
public unsafe class Stub_method_<xsl:value-of select="name"/>
{
    static Nerv35.Stub_signal.on_signal_ft on_signal_ = new Nerv35.Stub_signal.on_signal_ft(on_signal);
    static Nerv35.Stub_signal.on_cancel_ft on_cancel_ = new Nerv35.Stub_signal.on_cancel_ft(on_cancel);
    static ConcurrentDictionary&lt;IntPtr, Stub_method_<xsl:value-of select="name"/>&gt; dict = new ConcurrentDictionary&lt;IntPtr, Stub_method_<xsl:value-of select="name"/>&gt;();

    public static void on_signal(IntPtr _signal_,  Address _proxy_addr, IntPtr _sig_data_, UInt32 _sig_size, double _elapsed_time)
    {<xsl:if test="in_data!='(null)'">
        Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.<xsl:value-of select="in_data"/>* sig_data_ = (Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.<xsl:value-of select="in_data"/>*)_sig_data_;
        </xsl:if>
        dict[_signal_].isignal.on_signal(_proxy_addr,<xsl:if test="in_data!='(null)'"> ref *sig_data_,</xsl:if> _elapsed_time);
    }
    public static void on_cancel(IntPtr _signal_, Address _proxy_addr, Int32 _error)
    {
        dict[_signal_].isignal.on_cancel(_proxy_addr, _error);
    }

    public IntPtr signal_;
    public IStub_method_<xsl:value-of select="name"/> isignal;

    public Stub_method_<xsl:value-of select="name"/>(ref Nerv35.Stub _object_, IStub_method_<xsl:value-of select="name"/> _isignal)
    {
        signal_ = Nerv35.Stub_signal.alloc(<xsl:value-of select="code"/>, _object_.object_, on_signal_, on_cancel_);
        dict[signal_] = this;
        isignal = _isignal;
    }
    ~Stub_method_<xsl:value-of select="name"/>()
    {
        // Nerv35.Stub_signal.free(signal_);
        // API.nerv_finalize()에서 정리되지 않은 객체를 자동해제하므로
        // C#에서는 Nerv35.Stub_signal.free(signal_); 로 정리하지 않는다.
        // C#의 소멸자는 언제 호출될지 알수 없다.

        Stub_method_<xsl:value-of select="name"/> signal;
        dict.TryRemove(signal_, out signal);
    }
}
</xsl:template>



<xsl:template name ="template_event" >
  <xsl:variable name="in_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace_cs">
      <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="out_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace_cs">
      <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>



// proxy
public interface IProxy_method_<xsl:value-of select="name"/>
{
    void on_event(<xsl:if test="out_data!='(null)'">ref Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.<xsl:value-of select="out_data"/> _evt_data_,</xsl:if> double _elapsed_time);
    void on_unlink(Int32 _err);
}
public unsafe class Proxy_method_<xsl:value-of select="name"/>
{
    static Nerv35.Proxy_event.on_unlink_ft on_unlink_ = new Nerv35.Proxy_event.on_unlink_ft(on_unlink);
    static Nerv35.Proxy_event.on_event_ft on_event_   = new Nerv35.Proxy_event.on_event_ft(on_event);

    static ConcurrentDictionary&lt;IntPtr, Proxy_method_<xsl:value-of select="name"/>&gt; dict = new ConcurrentDictionary&lt;IntPtr, Proxy_method_<xsl:value-of select="name"/>&gt;();

    public static void on_event(IntPtr _event_,  IntPtr _evt_data_, UInt32 _evt_size, double _elapsed_time)
    {<xsl:if test="out_data!='(null)'">
        Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.<xsl:value-of select="out_data"/>* evt_data_ = (Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.<xsl:value-of select="out_data"/>*)_evt_data_;
        </xsl:if>
        dict[_event_].ievent.on_event(<xsl:if test="out_data!='(null)'"> ref *evt_data_,</xsl:if> _elapsed_time);
    }
    public static void on_unlink(IntPtr _event_, Int32 _error)
    {
        dict[_event_].ievent.on_unlink(_error);
    }

    public IntPtr event_;
    public IProxy_method_<xsl:value-of select="name"/> ievent;

    public Proxy_method_<xsl:value-of select="name"/>(ref Nerv35.Proxy _object_, IProxy_method_<xsl:value-of select="name"/> _ievent)
    {
        event_ = Nerv35.Proxy_event.alloc(<xsl:value-of select="code"/>, _object_.object_,on_unlink_,on_event_);
        dict[event_] = this;
        ievent = _ievent;
    }
    ~Proxy_method_<xsl:value-of select="name"/>()
    {
        // Nerv35.Proxy_event.free(event_);
        // API.nerv_finalize()에서 정리되지 않은 객체를 자동해제하므로
        // C#에서는 Nerv35.Proxy_event.free(event_); 로 정리하지 않는다.
        // C#의 소멸자는 언제 호출될지 알수 없다.

        Proxy_method_<xsl:value-of select="name"/> event_reference;
        dict.TryRemove(event_, out event_reference);
    }

    public Boolean is_linked()
    {
        return Nerv35.Proxy_event.is_linked(event_);
    }
    public IntPtr async_link(Double _limit_time)
    {
        IntPtr promise_ = Nerv35.Proxy_event.async_link( event_, _limit_time);
        return promise_;
    }
    public Int32 async_link_return( IntPtr _promise_, ref Double _time_)
    {
        Int32 err = Nerv35.Proxy_event.async_link_return(event_, _promise_, ref _time_);
        return err;
    }
    public Int32 link(Double _limit_time, ref Double _time_)
    {
        IntPtr promise_ = Nerv35.Proxy_event.async_link( event_, _limit_time);
        Int32 err = Nerv35.Proxy_event.async_link_return(event_, promise_, ref _time_);
        return err;
    }
    public void unlink()
    {
        Nerv35.Proxy_event.unlink(event_);
    }

}

// stub
public interface IStub_method_<xsl:value-of select="name"/>
{
    Int32 on_link(Address _proxy_addr, Double _packet_live_time, Double _packet_limit_time);
    void on_unlink(Address _proxy_addr, Int32 _err);
}

public unsafe class Stub_method_<xsl:value-of select="name"/>
{
    static Nerv35.Stub_event.on_link_ft on_link_     = new Nerv35.Stub_event.on_link_ft(on_link);
    static Nerv35.Stub_event.on_unlink_ft on_unlink_ = new Nerv35.Stub_event.on_unlink_ft(on_unlink);
    static ConcurrentDictionary&lt;IntPtr, Stub_method_<xsl:value-of select="name"/>&gt; dict = new ConcurrentDictionary&lt;IntPtr, Stub_method_<xsl:value-of select="name"/>&gt;();

    // nerv_stub_event delegate
    public static Int32 on_link(IntPtr _event_, Address _proxy_addr, Double _packet_live_time, Double _packet_limit_time)
    {
        Int32 err = dict[_event_].ievent.on_link(_proxy_addr, _packet_live_time, _packet_limit_time);
        return err;
    }
    public static void on_unlink(IntPtr _event_, Address _proxy_addr, Int32 _err)
    {
        dict[_event_].ievent.on_unlink(_proxy_addr, _err);
    }

    public IntPtr event_;
    public IStub_method_<xsl:value-of select="name"/> ievent;

    public Stub_method_<xsl:value-of select="name"/>(ref Nerv35.Stub _object_, IStub_method_<xsl:value-of select="name"/> _ievent)
    {
        event_ = Nerv35.Stub_event.alloc(<xsl:value-of select="code"/>, _object_.object_, on_link_, on_unlink_);
        dict[event_] = this;
        ievent = _ievent;
    }
    ~Stub_method_<xsl:value-of select="name"/>()
    {
        // Nerv35.Stub_event.free(event_);
        // API.nerv_finalize()에서 정리되지 않은 객체를 자동해제하므로
        // C#에서는 Nerv35.Stub_event.free(event_); 로 정리하지 않는다.
        // C#의 소멸자는 언제 호출될지 알수 없다.

        Stub_method_<xsl:value-of select="name"/> event_reference;
        dict.TryRemove(event_, out event_reference);
    }

    public void unlink(Address _proxy_addr)
    {
        Nerv35.Stub_event.unlink(event_, _proxy_addr);
    }

    public void unlink_all_proxy()
    {
        Nerv35.Stub_event.unlink_all_proxy(event_);
    }

    public Int32 notify(<xsl:if test="out_data!='(null)'">ref Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.<xsl:value-of select="out_data"/> _evt_data_</xsl:if>)
    {
        <xsl:if test="out_data!='(null)'">
        UInt32 evt_size = (UInt32)_evt_data_.fit_size();
        fixed ( Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.<xsl:value-of select="out_data"/>* evt_data_ = &amp;_evt_data_)
        {
            Int32 err = Nerv35.Stub_event.notify(event_, (IntPtr)evt_data_, evt_size);
            return err;
        }
        </xsl:if>
        <xsl:if test="out_data='(null)'">
        Int32 err = Nerv35.Stub_event.notify_nodata(event_);
        return err;
        </xsl:if>
    }
}

</xsl:template>


</xsl:stylesheet>
