<?xml version="1.0" encoding="utf-8"?>
<!-- copyright@ 2020, Waregram , All right Reserved. -->
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:include href="../xsl_util.xsl"/>
<xsl:output method="text" encoding="utf-8" indent="yes" />
<xsl:template match="/">
/*******************************************************************************
Component Name: <xsl:value-of select="/nerv_icde/component/name"/>

    // 구조체 크기가 큰경우 스택오버플로우가 발생하기 때문에
    // Marshal.AllocHGlobal(size) Marshal.FreeHGlobal(IntPtr)
    // 힙 할당해제함수를 이용하는 안전 실행 코드가 자동 생성된다.
    // 만약 구조체 크기가 작다면 다음처럼 간결한 코드가 가능하고 빠르다.
    {
        Nerv35.Data.Domain.Struct data = new Nerv35.Data.Domain.Struct;
        Int32 err = nerv_object.method.transaction(ref data);
    }

*******************************************************************************/
using Nerv35;
using System;
using System.Threading;
using System.Runtime.InteropServices;<xsl:text/>
  <xsl:for-each select="/nerv_icde/methoddomain/row">
    <xsl:for-each select="ref_data_domain/row">
      <xsl:variable name="data_domain_namespace">
        <xsl:call-template name="get_data_domain_namespace_cs">
          <xsl:with-param name="data_domain_id" select="data_domain_id" />
        </xsl:call-template>
      </xsl:variable>
<!-- using Nerv35.Data<xsl:value-of select="$data_domain_namespace"/>; -->
    </xsl:for-each>
  </xsl:for-each>

public class Nerv_component_<xsl:value-of select="/nerv_icde/component/name"/>
#if UNITY_64
    : UnityEngine.MonoBehaviour
#endif
{<xsl:text/>
  <xsl:for-each select="/nerv_icde/component/stub_list/row/object_list/row">
<xsl:text>
    </xsl:text><xsl:value-of select="../../class_name"/><xsl:text> </xsl:text>stub_<xsl:value-of select="object_name"/>;<xsl:text/>
  </xsl:for-each>
  <xsl:for-each select="/nerv_icde/component/proxy_list/row/object_list/row">
<xsl:text>
    </xsl:text><xsl:value-of select="../../class_name"/><xsl:text> </xsl:text>proxy_<xsl:value-of select="object_name"/>;<xsl:text/>
  </xsl:for-each>
/******************************************************************************/
    public void stub_start ()
    {
        UInt16 port;
        Int32 err;
<xsl:text/>
  <xsl:for-each select="/nerv_icde/component/stub_list/row/object_list/row">
<xsl:text>
        </xsl:text>stub_<xsl:value-of select="object_name"/> = new <xsl:value-of select="../../class_name"/>();<xsl:text/>
  </xsl:for-each>
  <xsl:for-each select="/nerv_icde/component/proxy_list/row/object_list/row">
<xsl:text>
        </xsl:text>proxy_<xsl:value-of select="object_name"/> = new <xsl:value-of select="../../class_name"/>();<xsl:text/>
  </xsl:for-each>
<xsl:text>
</xsl:text>
  <xsl:for-each select="/nerv_icde/component/stub_list/row/object_list/row">
    <xsl:variable name="method_domain_path">
      <xsl:call-template name="get_method_domain_path">
        <xsl:with-param name="mathod_domain_id" select="../../method_domain_id" />
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="method_domain_namespace">
      <xsl:call-template name="replace">
        <xsl:with-param name="text" select="$method_domain_path" />
        <xsl:with-param name="replace" select="'/'" />
        <xsl:with-param name="by" select="'.'" />
      </xsl:call-template>
    </xsl:variable>
<xsl:text>
        </xsl:text>port = 0; // Nerv35.Method.<xsl:value-of select="substring($method_domain_namespace,2)"/>.Standard.port;
        err = stub_<xsl:value-of select="object_name"/>.stub.start("<xsl:value-of select="object_name"/>", ref port);
        Output.Log(string.Format(
            "{0:D} = stub_<xsl:value-of select="object_name"/>.stub.start(port={1:D})",
            err, port));<xsl:text/>
  </xsl:for-each>
    }
/******************************************************************************/
    public void proxy_start ()
    {
        Int32 err;
        Address stub_addr;<xsl:text/>
  <xsl:for-each select="/nerv_icde/component/proxy_list/row/object_list/row">
    <xsl:variable name="method_domain_path">
      <xsl:call-template name="get_method_domain_path">
        <xsl:with-param name="mathod_domain_id" select="../../method_domain_id" />
      </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="method_domain_namespace">
      <xsl:call-template name="replace">
        <xsl:with-param name="text" select="$method_domain_path" />
        <xsl:with-param name="replace" select="'/'" />
        <xsl:with-param name="by" select="'.'" />
      </xsl:call-template>
    </xsl:variable>
<xsl:text>
        </xsl:text>
        if( ! proxy_<xsl:value-of select="object_name"/>.proxy.is_started()) {
            stub_addr.ip = 0;
            stub_addr.port = 0;
            int n = (int)Util.get_address(
                "<xsl:value-of select="object_name"/>",ref stub_addr);
            if( n > 0) {
                err = proxy_<xsl:value-of select="object_name"/>.proxy.start(stub_addr);
                Output.Log(string.Format(
                    "{0:D} = proxy_<xsl:value-of select="object_name"/>."+
                    "proxy.start({1},{2:D})",
                    err, Util.inet_ntoa(stub_addr.ip), stub_addr.port));
            }
        }<xsl:text/>
  </xsl:for-each>
    }<xsl:text/>
  <xsl:for-each select="/nerv_icde/component/proxy_list/row/object_list/row">
    <xsl:variable name="method_domain_id" select="../../method_domain_id" />
    <xsl:variable name="interface_name" select="../../interface_name" />
    <xsl:variable name="object_name" select="object_name" />
    <xsl:for-each select="/nerv_icde/methoddomain/row[id=$method_domain_id]">
      <xsl:for-each select="interface/row[name=$interface_name]">
        <xsl:for-each select="membermethod/row">
          <xsl:variable name="method_name" select="method_name" />
          <xsl:for-each select="../../../../method/row[name=$method_name]">
            <xsl:if test="type='INFORMATION' and out_data!='(null)'">
              <xsl:variable name="out_data_domain_namespace">
                <xsl:call-template name="get_data_domain_namespace_cs">
                  <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
                </xsl:call-template>
              </xsl:variable>
<xsl:text>
/******************************************************************************/
    </xsl:text>public unsafe void
        <xsl:value-of select="$object_name"/>_<xsl:value-of select="$method_name"/>_read()
    {
        int size = Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.
            <xsl:value-of select="out_data"/>._size;
        IntPtr ptr = Marshal.AllocHGlobal(size);
        ref Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.
            <xsl:value-of select="out_data"/> inf_data =
                ref *(Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.
                <xsl:value-of select="out_data"/>*)ptr;
        Int32 err = proxy_<xsl:value-of select="$object_name"/>.<xsl:value-of select="$method_name"/>.
            read(ref inf_data);
        if( err == 0 ) inf_data.print("<xsl:value-of select="out_data"/>");
        Marshal.FreeHGlobal(ptr);
    }<xsl:text/>
            </xsl:if>
          </xsl:for-each>
        </xsl:for-each>
      </xsl:for-each>
    </xsl:for-each>
  </xsl:for-each>
/******************************************************************************/
    public void proxy_read()
    {<xsl:text/>
  <xsl:for-each select="/nerv_icde/component/proxy_list/row/object_list/row">
    <xsl:variable name="method_domain_id" select="../../method_domain_id" />
    <xsl:variable name="interface_name" select="../../interface_name" />
    <xsl:variable name="object_name" select="object_name" />
    <xsl:for-each select="/nerv_icde/methoddomain/row[id=$method_domain_id]">
      <xsl:for-each select="interface/row[name=$interface_name]">
        <xsl:for-each select="membermethod/row">
          <xsl:variable name="method_name" select="method_name" />
          <xsl:for-each select="../../../../method/row[name=$method_name]">
            <xsl:if test="type='INFORMATION' and out_data!='(null)'">
<xsl:text>
        </xsl:text><xsl:value-of select="$object_name"/>_<xsl:value-of select="$method_name"/>_read();<xsl:text/>
            </xsl:if>
          </xsl:for-each>
        </xsl:for-each>
      </xsl:for-each>
    </xsl:for-each>
  </xsl:for-each>
    }<xsl:text/>
  <xsl:for-each select="/nerv_icde/component/proxy_list/row/object_list/row">
    <xsl:variable name="method_domain_id" select="../../method_domain_id" />
    <xsl:variable name="interface_name" select="../../interface_name" />
    <xsl:variable name="object_name" select="object_name" />
    <xsl:for-each select="/nerv_icde/methoddomain/row[id=$method_domain_id]">
      <xsl:for-each select="interface/row[name=$interface_name]">
        <xsl:for-each select="membermethod/row">
          <xsl:variable name="method_name" select="method_name" />
          <xsl:for-each select="../../../../method/row[name=$method_name]">
            <xsl:variable name="in_data_domain_namespace">
              <xsl:call-template name="get_data_domain_namespace_cs">
                <xsl:with-param name="data_domain_id" select="in_data_domain_id"/>
              </xsl:call-template>
            </xsl:variable>
            <xsl:variable name="out_data_domain_namespace">
              <xsl:call-template name="get_data_domain_namespace_cs">
                <xsl:with-param name="data_domain_id" select="out_data_domain_id"/>
              </xsl:call-template>
            </xsl:variable>
            <xsl:variable name="in_data_domain_namespace_dot" select="substring($in_data_domain_namespace,2)"/>
            <xsl:variable name="out_data_domain_namespace_dot" select="substring($out_data_domain_namespace,2)"/>
            <xsl:choose>
              <xsl:when test="type='CALL'">
<xsl:text>
/******************************************************************************/
    </xsl:text>public unsafe void <xsl:value-of select="$object_name"/>_<xsl:value-of select="$method_name"/>_call()
    {
        Double tm = 0;<xsl:text/>
                <xsl:if test="in_data='(null)' and out_data='(null)'">
        err = <xsl:value-of select="$object_name"/>.<xsl:value-of select="$method_name"/>.call(1.0, ref tm);<xsl:text/>
                </xsl:if>
                <xsl:if test="in_data!='(null)' and out_data='(null)'">
<xsl:text>
        </xsl:text>
        int cal_size = Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.
            <xsl:value-of select="in_data"/>._size;
        IntPtr cal_ptr = Marshal.AllocHGlobal(cal_size);
        ref Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.
            <xsl:value-of select="in_data"/> cal_data =
                ref *(Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.
                    <xsl:value-of select="in_data"/>*)cal_ptr;
        cal_data.set(0);
        Int32 err = proxy_<xsl:value-of select="$object_name"/>.<xsl:value-of select="$method_name"/>.
            call(ref cal_data, 1.0, ref tm);
        Marshal.FreeHGlobal(cal_ptr);<xsl:text/>
                </xsl:if>
                <xsl:if test="in_data='(null)' and out_data!='(null)'">
<xsl:text>
        </xsl:text>
        int ret_size = Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.
            <xsl:value-of select="out_data"/>._size;
        IntPtr ret_ptr = Marshal.AllocHGlobal(ret_size);
        ref Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.
            <xsl:value-of select="out_data"/> ret_data =
                ref *(Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.
                    <xsl:value-of select="out_data"/>*)ret_ptr;
        Int32 err = proxy_<xsl:value-of select="$object_name"/>.<xsl:value-of select="$method_name"/>.
            call(ref ret_data, 1.0, ref tm);
        if(err==0) ret_data.print("<xsl:value-of select="out_data"/>");
        Marshal.FreeHGlobal(ret_ptr);<xsl:text/>
                </xsl:if>
                <xsl:if test="in_data!='(null)' and out_data!='(null)'">
<xsl:text>
        </xsl:text>
        int cal_size = Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.
            <xsl:value-of select="in_data"/>._size;
        int ret_size = Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.
            <xsl:value-of select="out_data"/>._size;
        IntPtr cal_ptr = Marshal.AllocHGlobal(cal_size);
        IntPtr ret_ptr = Marshal.AllocHGlobal(ret_size);
        ref Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.
            <xsl:value-of select="in_data"/> cal_data =
                ref *(Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.
                    <xsl:value-of select="in_data"/>*)cal_ptr;
        ref Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.
            <xsl:value-of select="out_data"/> ret_data =
                ref *(Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.
                    <xsl:value-of select="out_data"/>*)ret_ptr;
        cal_data.set(0);
        Int32 err = proxy_<xsl:value-of select="$object_name"/>.<xsl:value-of select="$method_name"/>.
            call(ref cal_data, ref ret_data, 1.0, ref tm);
        if(err==0) ret_data.print("<xsl:value-of select="out_data"/>");
        Marshal.FreeHGlobal(cal_ptr);
        Marshal.FreeHGlobal(ret_ptr);<xsl:text/>
                </xsl:if>
    }<xsl:text/>
              </xsl:when>
              <xsl:when test="type='SIGNAL'">
<xsl:text>
/******************************************************************************/
    </xsl:text>public unsafe void <xsl:value-of select="$object_name"/>_<xsl:value-of select="$method_name"/>_signal()
    {
        Double tm = 0;<xsl:text/>
                <xsl:if test="in_data='(null)'">
        <xsl:value-of select="$object_name"/>.<xsl:value-of select="$method_name"/>.signal( 1.0, ref tm);<xsl:text/>
                </xsl:if>
                <xsl:if test="in_data!='(null)'">
<xsl:text>
        </xsl:text>int size = Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.
            <xsl:value-of select="in_data"/>._size;
        IntPtr ptr = Marshal.AllocHGlobal(size);
        ref Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.
            <xsl:value-of select="in_data"/> sig_data =
                ref *(Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.
                    <xsl:value-of select="in_data"/>*)ptr;
        sig_data.set(0);
        Int32 err = proxy_<xsl:value-of select="$object_name"/>.<xsl:value-of select="$method_name"/>.
            signal(ref sig_data, 1.0, ref tm);
        Marshal.FreeHGlobal(ptr);<xsl:text/>
                </xsl:if>
    }<xsl:text/>
              </xsl:when>
              <xsl:when test="type='COMMAND'">
<xsl:text>
/******************************************************************************/
    </xsl:text>public unsafe void <xsl:value-of select="$object_name"/>_<xsl:value-of select="$method_name"/>_command()
    {<xsl:text/>
                <xsl:if test="in_data='(null)'">
        <xsl:value-of select="$object_name"/>.<xsl:value-of select="$method_name"/>.write();<xsl:text/>
                </xsl:if>
                <xsl:if test="in_data!='(null)'">
<xsl:text>
        </xsl:text>int size = Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.
            <xsl:value-of select="in_data"/>._size;
        IntPtr ptr = Marshal.AllocHGlobal(size);
        ref Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.
            <xsl:value-of select="in_data"/> cmd_data =
                ref *(Nerv35.Data<xsl:value-of select="$in_data_domain_namespace"/>.
                    <xsl:value-of select="in_data"/>*)ptr;
        cmd_data.set(0);
        Int32 err = proxy_<xsl:value-of select="$object_name"/>.<xsl:value-of select="$method_name"/>.
            write(ref cmd_data);
        Marshal.FreeHGlobal(ptr);<xsl:text/>
                </xsl:if>
    }<xsl:text/>
              </xsl:when>
            </xsl:choose>
          </xsl:for-each>
        </xsl:for-each>
      </xsl:for-each>
    </xsl:for-each>
  </xsl:for-each>
/******************************************************************************/
    public void proxy_send()
    {<xsl:text/>
  <xsl:for-each select="/nerv_icde/component/proxy_list/row/object_list/row">
    <xsl:variable name="method_domain_id" select="../../method_domain_id" />
    <xsl:variable name="interface_name" select="../../interface_name" />
    <xsl:variable name="object_name" select="object_name" />
    <xsl:for-each select="/nerv_icde/methoddomain/row[id=$method_domain_id]">
      <xsl:for-each select="interface/row[name=$interface_name]">
        <xsl:for-each select="membermethod/row">
          <xsl:variable name="method_name" select="method_name" />
          <xsl:for-each select="../../../../method/row[name=$method_name]">
            <xsl:choose>
              <xsl:when test="type='CALL'">
<xsl:text>
        </xsl:text><xsl:value-of select="$object_name"/>_<xsl:value-of select="$method_name"/>_call();<xsl:text/>
              </xsl:when>
              <xsl:when test="type='SIGNAL'">
<xsl:text>
        </xsl:text><xsl:value-of select="$object_name"/>_<xsl:value-of select="$method_name"/>_signal();<xsl:text/>
              </xsl:when>
              <xsl:when test="type='COMMAND'">
<xsl:text>
        </xsl:text><xsl:value-of select="$object_name"/>_<xsl:value-of select="$method_name"/>_command();<xsl:text/>
              </xsl:when>
            </xsl:choose>
          </xsl:for-each>
        </xsl:for-each>
      </xsl:for-each>
    </xsl:for-each>
  </xsl:for-each>
    }<xsl:text/>
  <xsl:for-each select="/nerv_icde/component/stub_list/row/object_list/row">
    <xsl:variable name="method_domain_id" select="../../method_domain_id" />
    <xsl:variable name="interface_name" select="../../interface_name" />
    <xsl:variable name="object_name" select="object_name" />
    <xsl:for-each select="/nerv_icde/methoddomain/row[id=$method_domain_id]">
      <xsl:for-each select="interface/row[name=$interface_name]">
        <xsl:for-each select="membermethod/row">
          <xsl:variable name="method_name" select="method_name" />
          <xsl:for-each select="../../../../method/row[name=$method_name]">
            <xsl:variable name="in_data_domain_namespace">
              <xsl:call-template name="get_data_domain_namespace_cs">
                <xsl:with-param name="data_domain_id" select="in_data_domain_id"/>
              </xsl:call-template>
            </xsl:variable>
            <xsl:variable name="out_data_domain_namespace">
              <xsl:call-template name="get_data_domain_namespace_cs">
                <xsl:with-param name="data_domain_id" select="out_data_domain_id"/>
              </xsl:call-template>
            </xsl:variable>
            <xsl:variable name="in_data_domain_namespace_dot" select="substring($in_data_domain_namespace,2)"/>
            <xsl:variable name="out_data_domain_namespace_dot" select="substring($out_data_domain_namespace,2)"/>
            <xsl:choose>
              <xsl:when test="type='EVENT'">
<xsl:text>
/******************************************************************************/
    </xsl:text>public unsafe void <xsl:value-of select="$object_name"/>_<xsl:value-of select="$method_name"/>_event()
    {<xsl:text/>
                <xsl:if test="out_data='(null)'">
<xsl:text>
        Int32 err = </xsl:text><xsl:value-of select="$object_name"/>.<xsl:value-of select="$method_name"/>.notify();<xsl:text/>
                </xsl:if>
                <xsl:if test="out_data!='(null)'">
<xsl:text>
        </xsl:text>int size = Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.
            <xsl:value-of select="out_data"/>._size;
        IntPtr ptr = Marshal.AllocHGlobal(size);
        ref Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.
            <xsl:value-of select="out_data"/> evt_data =
                ref *(Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.
                    <xsl:value-of select="out_data"/>*)ptr;
        evt_data.set(0);
        Int32 err = stub_<xsl:value-of select="$object_name"/>.<xsl:value-of select="$method_name"/>.
            notify(ref evt_data);
        Marshal.FreeHGlobal(ptr);<xsl:text/>
                </xsl:if>
    }<xsl:text/>
              </xsl:when>
              <xsl:when test="type='INFORMATION'">
<xsl:text>
/******************************************************************************/
    </xsl:text>public unsafe void <xsl:value-of select="$object_name"/>_<xsl:value-of select="$method_name"/>_information()
    {<xsl:text/>
                <xsl:if test="out_data='(null)'">
<xsl:text>
        Int32 err = </xsl:text><xsl:value-of select="$object_name"/>.<xsl:value-of select="$method_name"/>.write();<xsl:text/>
                </xsl:if>
                <xsl:if test="out_data!='(null)'">
<xsl:text>
        </xsl:text>int size = Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.
            <xsl:value-of select="out_data"/>._size;
        IntPtr ptr = Marshal.AllocHGlobal(size);
        ref Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.
            <xsl:value-of select="out_data"/> inf_data =
                ref *(Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.
                    <xsl:value-of select="out_data"/>*)ptr;
        inf_data.set(0);
        Int32 err = stub_<xsl:value-of select="$object_name"/>.<xsl:value-of select="$method_name"/>.
            write(ref inf_data);
        Marshal.FreeHGlobal(ptr);<xsl:text/>
                </xsl:if>
    }<xsl:text/>
              </xsl:when>

              <xsl:when test="type='DATAGRAM'">
<xsl:text>
/******************************************************************************/
    </xsl:text>public unsafe void <xsl:value-of select="$object_name"/>_<xsl:value-of select="$method_name"/>_datagram()
    {<xsl:text/>
                <xsl:if test="out_data='(null)'">
<xsl:text>
        Int32 err = </xsl:text><xsl:value-of select="$object_name"/>.<xsl:value-of select="$method_name"/>.broadcast();<xsl:text/>
                </xsl:if>
                <xsl:if test="out_data!='(null)'">
<xsl:text>
        </xsl:text>int size = Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.
            <xsl:value-of select="out_data"/>._size;
        IntPtr ptr = Marshal.AllocHGlobal(size);
        ref Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.
            <xsl:value-of select="out_data"/> dtg_data =
                ref *(Nerv35.Data<xsl:value-of select="$out_data_domain_namespace"/>.
                    <xsl:value-of select="out_data"/>*)ptr;
        dtg_data.set(0);
        Int32 err = stub_<xsl:value-of select="$object_name"/>.<xsl:value-of select="$method_name"/>.
            broadcast(ref dtg_data);
        Marshal.FreeHGlobal(ptr);<xsl:text/>
                </xsl:if>
    }<xsl:text/>
              </xsl:when>
            </xsl:choose>
          </xsl:for-each>
        </xsl:for-each>
      </xsl:for-each>
    </xsl:for-each>
  </xsl:for-each>
  <xsl:for-each select="/nerv_icde/component/stub_list/row/object_list/row">
    <xsl:variable name="method_domain_id" select="../../method_domain_id" />
    <xsl:variable name="interface_name" select="../../interface_name" />
    <xsl:variable name="object_name" select="object_name" />
    <xsl:for-each select="/nerv_icde/methoddomain/row[id=$method_domain_id]">
      <xsl:for-each select="interface/row[name=$interface_name]">
        <xsl:for-each select="membermethod/row">
          <xsl:variable name="method_name" select="method_name" />
          <xsl:for-each select="../../../../method/row[name=$method_name]">
            <xsl:if test="type='COMMAND' and in_data!='(null)'">
              <xsl:variable name="data_domain_namespace">
                <xsl:call-template name="get_data_domain_namespace_cs">
                  <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
                </xsl:call-template>
              </xsl:variable>
<xsl:text>
/******************************************************************************/
    </xsl:text>public unsafe void <xsl:value-of select="$object_name"/>_<xsl:value-of select="$method_name"/>_read()
    {
        int size = Nerv35.Data<xsl:value-of select="$data_domain_namespace"/>.
            <xsl:value-of select="in_data"/>._size;
        IntPtr ptr = Marshal.AllocHGlobal(size);
        ref Nerv35.Data<xsl:value-of select="$data_domain_namespace"/>.
            <xsl:value-of select="in_data"/> cmd_data =
                ref *(Nerv35.Data<xsl:value-of select="$data_domain_namespace"/>.
                    <xsl:value-of select="in_data"/>*)ptr;
        Int32 err = stub_<xsl:value-of select="$object_name"/>.<xsl:value-of select="$method_name"/>.
            read(stub_<xsl:value-of select="$object_name"/>.proxy_addr, ref cmd_data);
        if( err == 0 ) cmd_data.print("<xsl:value-of select="in_data"/>");
        Marshal.FreeHGlobal(ptr);
    }<xsl:text/>

            </xsl:if>
          </xsl:for-each>
        </xsl:for-each>
      </xsl:for-each>
    </xsl:for-each>
  </xsl:for-each>
/******************************************************************************/
    public void stub_read()
    {<xsl:text/>
  <xsl:for-each select="/nerv_icde/component/stub_list/row/object_list/row">
    <xsl:variable name="method_domain_id" select="../../method_domain_id" />
    <xsl:variable name="interface_name" select="../../interface_name" />
    <xsl:variable name="object_name" select="object_name" />
    <xsl:for-each select="/nerv_icde/methoddomain/row[id=$method_domain_id]">
      <xsl:for-each select="interface/row[name=$interface_name]">
        <xsl:for-each select="membermethod/row">
          <xsl:variable name="method_name" select="method_name" />
          <xsl:for-each select="../../../../method/row[name=$method_name]">
            <xsl:if test="type='COMMAND' and in_data!='(null)'">
<xsl:text>
        </xsl:text><xsl:value-of select="$object_name"/>_<xsl:value-of select="$method_name"/>_read();<xsl:text/>
            </xsl:if>
          </xsl:for-each>
        </xsl:for-each>
      </xsl:for-each>
    </xsl:for-each>
  </xsl:for-each>
    }<xsl:text/>
/******************************************************************************/
    public void stub_send()
    {<xsl:text/>
  <xsl:for-each select="/nerv_icde/component/stub_list/row/object_list/row">
    <xsl:variable name="method_domain_id" select="../../method_domain_id" />
    <xsl:variable name="interface_name" select="../../interface_name" />
    <xsl:variable name="object_name" select="object_name" />
    <xsl:for-each select="/nerv_icde/methoddomain/row[id=$method_domain_id]">
      <xsl:for-each select="interface/row[name=$interface_name]">
        <xsl:for-each select="membermethod/row">
          <xsl:variable name="method_name" select="method_name" />
          <xsl:for-each select="../../../../method/row[name=$method_name]">
            <xsl:choose>
              <xsl:when test="type='EVENT'">
<xsl:text>
        </xsl:text><xsl:value-of select="$object_name"/>_<xsl:value-of select="$method_name"/>_event();<xsl:text/>
              </xsl:when>
              <xsl:when test="type='INFORMATION'">
<xsl:text>
        </xsl:text><xsl:value-of select="$object_name"/>_<xsl:value-of select="$method_name"/>_information();<xsl:text/>
              </xsl:when>
              <xsl:when test="type='DATAGRAM'">
<xsl:text>
        </xsl:text><xsl:value-of select="$object_name"/>_<xsl:value-of select="$method_name"/>_datagram();<xsl:text/>
              </xsl:when>
            </xsl:choose>
          </xsl:for-each>
        </xsl:for-each>
      </xsl:for-each>
    </xsl:for-each>
  </xsl:for-each>
    }
/******************************************************************************/
    public void Start ()
    {
        Output.Log("Nerv_component_<xsl:value-of select="/nerv_icde/component/name"/>.Start()");

        Util.initialize_default();

        stub_start();
    }
/******************************************************************************/
    public void Update()
    {
        // Output.Log("Update()");
        proxy_start();

        proxy_read();
        proxy_send();

        stub_read();
        stub_send();
    }
/******************************************************************************/
    public void OnDisable()
    {
        Util.finalize ();
        Output.Log("Nerv_component_<xsl:value-of select="/nerv_icde/component/name"/>.OnDisable()");
    }
/******************************************************************************/
    static void Main(string[] args)
    {
        Console.OutputEncoding = System.Text.Encoding.UTF8;
        Console.InputEncoding = System.Text.Encoding.UTF8;

        Nerv_component_<xsl:value-of select="/nerv_icde/component/name"/><xsl:text> </xsl:text><xsl:value-of select="/nerv_icde/component/name"/> = new Nerv_component_<xsl:value-of select="/nerv_icde/component/name"/>();

        <xsl:value-of select="/nerv_icde/component/name"/>.Start();

        for (int i = 0; i &lt; 1000000; i++)
        {
            <xsl:value-of select="/nerv_icde/component/name"/>.Update();
            Thread.Sleep(1000);
        }

        <xsl:value-of select="/nerv_icde/component/name"/>.OnDisable();
    }
/******************************************************************************/
}
</xsl:template>
</xsl:stylesheet>
