<?xml version="1.0" encoding="utf-8"?>
<!-- copyright@ 2020, Waregram , All right Reserved. -->
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:include href="../../../../xsl_util.xsl"/>

<xsl:output method="text" encoding="utf-8" indent="yes"/>

<xsl:param name = "export_proxy_class_id" />

<xsl:template match="/">
  <xsl:for-each select="/nerv_icde/component/proxy_list/row[id=$export_proxy_class_id]">
    <xsl:variable name="method_domain_id" select="method_domain_id" />
    <xsl:variable name="interface_name" select="interface_name" />
    <xsl:variable name="class_name" select="class_name" />

#include "<xsl:value-of select="$class_name"/>.h"

<xsl:value-of select="$class_name"/>::<xsl:value-of select="$class_name"/>()
{
}

<xsl:value-of select="$class_name"/>::~<xsl:value-of select="$class_name"/>()
{
}

void <xsl:value-of select="$class_name"/>::on_connect()
{
    PyGILState_STATE state = PyGILState_Ensure();
    this->get_override("on_connect")();
    PyGILState_Release(state);
}

void <xsl:value-of select="$class_name"/>::on_disconnect()
{
    PyGILState_STATE state = PyGILState_Ensure();
    this->get_override("on_disconnect")();
    PyGILState_Release(state);
}

    <xsl:for-each select="/nerv_icde/methoddomain/row[id=$method_domain_id]">
      <xsl:for-each select="interface/row[name=$interface_name]">
        <xsl:for-each select="membermethod/row">
          <xsl:variable name="method_name" select="method_name" />
          <xsl:for-each select="../../../../method/row[name=$method_name]">

            <xsl:variable name="in_data_domain_namespace">
              <xsl:call-template name="get_data_domain_namespace">
                <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
              </xsl:call-template>
            </xsl:variable>
            <xsl:variable name="out_data_domain_namespace">
              <xsl:call-template name="get_data_domain_namespace">
                <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
              </xsl:call-template>
            </xsl:variable>
            <xsl:choose>
              <xsl:when test="type='CALL'">
              </xsl:when>
              <xsl:when test="type='SIGNAL'">
              </xsl:when>

              <xsl:when test="type='EVENT'">
void <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_unlink(Nerv35::nerv_err_t _err)
{
    PyGILState_STATE state = PyGILState_Ensure();
    this->get_override("on_<xsl:value-of select="$method_name"/>_unlink")(_err);
    PyGILState_Release(state);
}
                <xsl:choose >
                  <xsl:when test="out_data='(null)'">
void <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>( double _elapsed_time)
{
    PyGILState_STATE state = PyGILState_Ensure();
    this->get_override("on_<xsl:value-of select="$method_name"/>")(_elapsed_time);
    PyGILState_Release(state);
}
                  </xsl:when>
                  <xsl:otherwise>
void <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>(const Nerv35::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _evt_data_, double _elapsed_time)
{
    PyGILState_STATE state = PyGILState_Ensure();
    this->get_override("on_<xsl:value-of select="$method_name"/>")(_evt_data_, _elapsed_time);
    PyGILState_Release(state);
}
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:when>


              <xsl:when test="type='COMMAND'">
void <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_unlink(Nerv35::nerv_err_t _err)
{
    PyGILState_STATE state = PyGILState_Ensure();
    this->get_override("on_<xsl:value-of select="$method_name"/>_unlink")(_err);
    PyGILState_Release(state);
}
              </xsl:when>


              <xsl:when test="type='INFORMATION'">
void <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_unlink(Nerv35::nerv_err_t _err)
{
    PyGILState_STATE state = PyGILState_Ensure();
    this->get_override("on_<xsl:value-of select="$method_name"/>_unlink")(_err);
    PyGILState_Release(state);
}
                <xsl:choose >
                  <xsl:when test="out_data='(null)'">
void <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_sync( Nerv35::Sync_model _sync_model, double _elapsed_time, double _limit_time )
{
    PyGILState_STATE state = PyGILState_Ensure();
    this->get_override("on_<xsl:value-of select="$method_name"/>_sync")((Nerv35::uint8_t)_sync_model, _elapsed_time, _limit_time);
    PyGILState_Release(state);
}
                  </xsl:when>
                  <xsl:otherwise>
void <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_sync( Nerv35::Sync_model _sync_model, double _elapsed_time, double _limit_time, const Nerv35::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _buffer_ )
{
    PyGILState_STATE state = PyGILState_Ensure();
    this->get_override("on_<xsl:value-of select="$method_name"/>_sync")((Nerv35::uint8_t)_sync_model, _elapsed_time, _limit_time, _buffer_);
    PyGILState_Release(state);
}
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:when>


              <xsl:when test="type='DATAGRAM'">
                <xsl:choose >
                  <xsl:when test="out_data='(null)'">
void <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_datagram( ::Nerv35::addr_t _stub_addr, double _elapsed_time)
{
    PyGILState_STATE state = PyGILState_Ensure();
    this->get_override("on_<xsl:value-of select="$method_name"/>_datagram")(_stub_addr, _elapsed_time);
    PyGILState_Release(state);
}
                  </xsl:when>
                  <xsl:otherwise>
void <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_datagram( ::Nerv35::addr_t _stub_addr, const Nerv35::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _buf_, double _elapsed_time)
{
    PyGILState_STATE state = PyGILState_Ensure();
    this->get_override("on_<xsl:value-of select="$method_name"/>_datagram")(_stub_addr, _buf_, _elapsed_time);
    PyGILState_Release(state);
}
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:when>
            </xsl:choose>
          </xsl:for-each>
        </xsl:for-each>
      </xsl:for-each>
    </xsl:for-each>
  </xsl:for-each>
</xsl:template>

</xsl:stylesheet>
