<?xml version="1.0" encoding="UTF-8"?>
<!-- copyright@ 2020, Waregram , All right Reserved. -->
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:include href="../../../../../xsl_util.xsl"/>

<xsl:output method="text" encoding="utf-8" indent="yes"/>

<xsl:param name = "export_method_domain_path" />

<xsl:template match="/">
<xsl:for-each select="nerv_icde/methoddomain/row">
<xsl:if test="path=$export_method_domain_path">
<xsl:variable name="open_namespace_path">
<xsl:call-template name="replace">
  <xsl:with-param name="text" select="path" />
  <xsl:with-param name="replace" select="'/'" />
  <xsl:with-param name="by" select="' { namespace '" />
</xsl:call-template>
</xsl:variable>
<xsl:variable name="close_namespace_path">
<xsl:call-template name="replace">
  <xsl:with-param name="text" select="path" />
  <xsl:with-param name="replace" select="'/'" />
  <xsl:with-param name="by" select="' */ } /* '" />
</xsl:call-template>
</xsl:variable>
<xsl:variable name="define_path">
<xsl:call-template name="replace">
  <xsl:with-param name="text" select="path" />
  <xsl:with-param name="replace" select="'/'" />
  <xsl:with-param name="by" select="'__'" />
</xsl:call-template>
</xsl:variable>
/****************************************************************************************************************************************************
    Document Header
    domain   : <xsl:value-of select="name"/>
    creator  : <xsl:value-of select="author"/>
    create date          : <xsl:value-of select="create-day"/>
    description          : <xsl:value-of select="description"/>
****************************************************************************************************************************************************/

#pragma once


/* Include Nerv Sdk Document ***********************************************************************************************************************/
#include &lt;Nerv35/Nerv.hh&gt;

#include &lt;stdio.h&gt;

/* Include Data Domains Document *****************************************************************************************************************/
<xsl:for-each select="ref_data_domain/row">
    <xsl:variable name="ref_domain_id" select="data_domain_id"/>
    <xsl:for-each select="/nerv_icde/datadomain/row">
        <xsl:if test="id=$ref_domain_id">
#include &lt;Nerv35/Data<xsl:value-of select="path"/>/<xsl:value-of select="name"/>.h&gt;
        </xsl:if>
    </xsl:for-each>
</xsl:for-each>


/* Domain Define ***********************************************************************************************************************************/

namespace Nerv35 { namespace Method <xsl:value-of select="$open_namespace_path" /> {

    using namespace Nerv35;

    /* Standard Domain Port Constance **************************************************************************************************************/
    const ::Nerv35::port_t c_standard_port = <xsl:value-of select="standard_port"/>;


    /* Method Defines ******************************************************************************************************************************/

<xsl:for-each select="method/row">
    <xsl:variable name="in_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
    </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="out_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
    </xsl:call-template>
    </xsl:variable>
    /********************************************************************************************************************************************
        Method       : <xsl:value-of select="name"/>
        Input        : <xsl:if test="in_data != '(null)'"><xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/></xsl:if>
        output       : <xsl:if test="out_data != '(null)'"><xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/></xsl:if>
        Method Type  : <xsl:value-of select="type"/>
        Code         : <xsl:value-of select="code"/>
        Writer       : <xsl:value-of select="../../author"/>
        Description  : <xsl:value-of select="../../description"/>
    ********************************************************************************************************************************************/
  <xsl:if test="type='INFORMATION'" ><xsl:call-template name="template_information" /></xsl:if>
  <xsl:if test="type='COMMAND'" ><xsl:call-template name="template_command" /></xsl:if>
  <xsl:if test="type='CALL'" ><xsl:call-template name="template_call" /></xsl:if>
  <xsl:if test="type='SIGNAL'" ><xsl:call-template name="template_signal" /></xsl:if>
  <xsl:if test="type='EVENT'" ><xsl:call-template name="template_event" /></xsl:if>
  <xsl:if test="type='DATAGRAM'" ><xsl:call-template name="template_datagram" /></xsl:if>
</xsl:for-each>


    /* Interface Defines ***************************************************************************************************************************/


<xsl:for-each select="interface/row">
    /********************************************************************************************************************************************
        Interface Name : <xsl:value-of select="name"/>
        Writer : <xsl:value-of select="../../author"/>
        Description : <xsl:value-of select="description"/>
    *******************************************************************************************************************************************/

    /* proxy */
    class proxy_interface_<xsl:value-of select="name"/>
    : virtual public ::Nerv35::Proxy<xsl:for-each select="membermethod/row">
    , virtual public proxy_method_<xsl:value-of select="method_name"/>
    </xsl:for-each>
    {
    };

    /* stub */
    class stub_interface_<xsl:value-of select="name"/>
    : virtual public ::Nerv35::Stub<xsl:for-each select="membermethod/row">
    , virtual public method_<xsl:value-of select="method_name"/>
    </xsl:for-each>
    {
    };
</xsl:for-each>

} } /* <xsl:value-of select="$close_namespace_path" /> */


</xsl:if>
</xsl:for-each>
</xsl:template>


<xsl:template name ="template_information" >
  <xsl:variable name="in_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="out_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>

    /* proxy */
    class proxy_method_<xsl:value-of select="name"/> : public ::Nerv35::Proxy_information
    {
    public :
        proxy_method_<xsl:value-of select="name"/>() : Proxy_information(<xsl:value-of select="code"/>) {}

        bool <xsl:value-of select="name"/>_is_connected() { return is_connected(); }

        ::Nerv35::nerv_err_t <xsl:value-of select="name"/>_link( bool _reliability = false, double _limit_time = 1.0, double* _time_ = 0)
        {
            return_promise_t return_object = async_link( _reliability , _limit_time );
            return async_link_return( return_object, _time_ );
        }

        void <xsl:value-of select="name"/>_unlink() { unlink(); }

        void <xsl:value-of select="name"/>_set_sync(bool _use_on_sync) { set_sync(_use_on_sync); }

        letter_case_t <xsl:value-of select="name"/>_see_history(size_t _size)
        {
            return see_history(_size);
        }

        void <xsl:value-of select="name"/>_set_history_size(size_t _history_size)
        {
            set_history_size( _history_size);
        }

        nerv_err_t <xsl:value-of select="name"/>_read( Nerv35::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _data_)
        {
            Nerv35::pksize_t size = sizeof(Nerv35::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>);
            return read(_data_, &amp;size );
        }

    private:

        virtual void on_unlink( nerv_err_t _err)
        {
            on_<xsl:value-of select="name"/>_unlink(_err);
        }

        virtual void on_sync(Sync_model  _sync_model,
                             double      _elapsed_time,
                             double      _limit_time,
                             const void* _buffer_,
                             pksize_t    _size)
        {
            on_<xsl:value-of select="name"/>_sync(_sync_model, _elapsed_time, _limit_time, (const Nerv35::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*)_buffer_ );
        }

        virtual void on_<xsl:value-of select="name"/>_unlink(nerv_err_t _err) = 0;

        virtual void on_<xsl:value-of select="name"/>_sync( Sync_model in_sync_model,
                                                double _elapsed_time,
                                                double _limit_time,
                                               const Nerv35::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _buffer_ ) = 0;
    };

    /* stub */
    class method_<xsl:value-of select="name"/> : public ::Nerv35::Stub_information
    {
    public:

        method_<xsl:value-of select="name"/>(int interval= 0) : Stub_information(<xsl:value-of select="code"/>) {}

        void <xsl:value-of select="name"/>_set_sync_model( Sync_model _sync_model, double _interval)
        {
            set_sync_model(_sync_model,_interval);
        }

        void <xsl:value-of select="name"/>_unlink( addr_t _proxy_addr )
        {
            unlink( _proxy_addr );
        }

        void <xsl:value-of select="name"/>_unlink_all_proxy()
        {
            unlink_all_proxy();
        }

        Nerv35::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* <xsl:value-of select="name"/>_get()
        {
            return (Nerv35::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*) get(sizeof(Nerv35::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>) );
        }

        void <xsl:value-of select="name"/>_cancel()
        {
            cancel();
        }

        nerv_err_t <xsl:value-of select="name"/>_post(Nerv35::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _data_)
        {
            ::Nerv35::pksize_t size = nerv_sizeof(_data_);
            return post( size);
        }

        nerv_err_t <xsl:value-of select="name"/>_write(Nerv35::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _data_)
        {
            ::Nerv35::pksize_t size = nerv_sizeof(_data_);
            return write( _data_, size);
        }

        letter_case_t <xsl:value-of select="name"/>_see_history(size_t _size)
        {
            return see_history( _size);
        }

        void <xsl:value-of select="name"/>_set_history_size(size_t _history_size)
        {
            set_history_size(_history_size);
        }

        nerv_err_t <xsl:value-of select="name"/>_read( Nerv35::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _data_)
        {
            Nerv35::pksize_t size = sizeof(Nerv35::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>);
            return read(_data_, &amp;size );
        }

    private:

        virtual nerv_err_t on_link( addr_t _proxy_addr,
                                    bool        _reliability,
                                    double      _packet_live_time,
                                    double      _packet_limit_time )
        {
            return on_<xsl:value-of select="name"/>_link( _proxy_addr, _reliability, _packet_live_time, _packet_limit_time );
        }

        virtual void on_unlink( addr_t _proxy_addr, nerv_err_t _err)
        {
            on_<xsl:value-of select="name"/>_unlink(_proxy_addr, _err);
        }

        virtual nerv_err_t on_<xsl:value-of select="name"/>_link(addr_t _proxy_addr,
                                                                 bool        _reliability,
                                                                 double      _packet_live_time,
                                                                 double      _packet_limit_time ) = 0;

        virtual void on_<xsl:value-of select="name"/>_unlink( addr_t _proxy_addr, nerv_err_t _err) = 0;


    };
</xsl:template>

<xsl:template name ="template_command" >
  <xsl:variable name="in_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="out_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
    /* proxy */
    class proxy_method_<xsl:value-of select="name"/> : public ::Nerv35::Proxy_command
    {
    public :
        proxy_method_<xsl:value-of select="name"/>() : Proxy_command(<xsl:value-of select="code"/>) {}

        bool <xsl:value-of select="name"/>_is_connected() { return is_connected(); }

        ::Nerv35::nerv_err_t <xsl:value-of select="name"/>_link( double _limit_time = 1.0, double* _time_ = 0)
        {
            return_promise_t return_object = async_link( _limit_time );
            return async_link_return( return_object, _time_ );
        }

        void <xsl:value-of select="name"/>_unlink() { unlink(); }

        void <xsl:value-of select="name"/>_set_sync_model( bool _reliability,
                                                           Nerv35::Sync_model _sync_model,
                                                           double _interval )
        {
            set_sync_model( _reliability, _sync_model, _interval);
        }

        Nerv35::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* <xsl:value-of select="name"/>_get()
        {
            return (Nerv35::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>*)get(sizeof(Nerv35::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>));
        }

        void <xsl:value-of select="name"/>_cancel()
        {
            cancel();
        }

        nerv_err_t <xsl:value-of select="name"/>_post(Nerv35::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _data_  )
        {
            ::Nerv35::pksize_t size = nerv_sizeof(_data_);
            return post(size);
        }

        nerv_err_t <xsl:value-of select="name"/>_write(const Nerv35::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _data_)
        {
            ::Nerv35::pksize_t size = nerv_sizeof(_data_);
            return write( _data_, size);
        }

        letter_case_t <xsl:value-of select="name"/>_see_history(size_t _size)
        {
            return see_history( _size);
        }

        void <xsl:value-of select="name"/>_set_history_size( size_t _history_size)
        {
            set_history_size( _history_size);
        }

        nerv_err_t <xsl:value-of select="name"/>_read( Nerv35::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _data_)
        {
            Nerv35::pksize_t size = sizeof(Nerv35::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>);
            return read(_data_, &amp;size );
        }

    private:

        virtual void on_unlink(nerv_err_t _err)
        {
            on_<xsl:value-of select="name"/>_unlink(_err);
        }

        virtual void on_<xsl:value-of select="name"/>_unlink(nerv_err_t _err) = 0;
    };

    /* stub */
    class method_<xsl:value-of select="name"/> : public ::Nerv35::Stub_command
    {
    public:

        method_<xsl:value-of select="name"/>(int interval= 0) : Stub_command(<xsl:value-of select="code"/>) {}

        void <xsl:value-of select="name"/>_unlink( addr_t _proxy_addr )
        {
            unlink( _proxy_addr );
        }

        void <xsl:value-of select="name"/>_unlink_all_proxy()
        {
            unlink_all_proxy();
        }

        void <xsl:value-of select="name"/>_set_sync(bool _use_on_sync) { set_sync(_use_on_sync); }

        letter_case_t <xsl:value-of select="name"/>_see_history( addr_t _proxy_addr, size_t _size)
        {
            return see_history(_proxy_addr, _size);
        }

        void <xsl:value-of select="name"/>_set_history_size( size_t _history_size)
        {
            set_history_size( _history_size);
        }

//        int <xsl:value-of select="name"/>_get_client_count()
//        {
//            return get_client_count();
//        }

        nerv_err_t <xsl:value-of select="name"/>_read(addr_t _proxy_addr, Nerv35::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _data_)
        {
            Nerv35::pksize_t size = sizeof(<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>);
            return read(_proxy_addr, _data_, &amp;size );
        }

    private:

        virtual void on_sync(addr_t _proxy_addr,
                             Sync_model  _sync_model,
                             double      _elapsed_time,
                             double      _limit_time,
                             const void* _buffer_,
                             pksize_t    _size)
        {
            on_<xsl:value-of select="name"/>_sync(_proxy_addr, _sync_model,_elapsed_time, _limit_time,(const Nerv35::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>*)_buffer_);
        }

        virtual nerv_err_t on_link( addr_t _proxy_addr,
                                       double _packet_live_time,
                                       double _packet_limit_time )
        {
            return on_<xsl:value-of select="name"/>_link( _proxy_addr, _packet_live_time, _packet_limit_time );
        }

        virtual void on_unlink( addr_t _proxy_addr, nerv_err_t _err)
        {
            on_<xsl:value-of select="name"/>_unlink(_proxy_addr, _err);
        }

        virtual nerv_err_t on_<xsl:value-of select="name"/>_link( addr_t _proxy_addr,
                                                                     double _packet_live_time,
                                                                     double _packet_limit_time ) = 0;

        virtual void on_<xsl:value-of select="name"/>_unlink(addr_t _proxy_addr, nerv_err_t _err) = 0;

        virtual void on_<xsl:value-of select="name"/>_sync( addr_t _proxy_addr,
                                                   Sync_model _sync_model,
                                                   double _elapsed_time,
                                                   double _limit_time,
                                                   const <xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _buffer_ ) = 0; //{}
    };
</xsl:template>

<xsl:template name ="template_call" >
  <xsl:variable name="in_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="out_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
    /* proxy */
    class proxy_method_<xsl:value-of select="name"/> : public ::Nerv35::Proxy_call
    {
    public :
        proxy_method_<xsl:value-of select="name"/>() : Proxy_call(<xsl:value-of select="code"/>) {}

        ::Nerv35::nerv_err_t <xsl:value-of select="name"/>(<xsl:if test="in_data!='(null)'"> Nerv35::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _cal_data_,</xsl:if><xsl:if test="out_data!='(null)'"> Nerv35::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _return_data_,</xsl:if> double _limit_time = 1.0, double* _time_ = 0)
        {
            <xsl:if test="in_data!='(null)'">::Nerv35::pksize_t in_size = nerv_sizeof(_cal_data_);</xsl:if>
            <xsl:if test="out_data!='(null)'">::Nerv35::pksize_t out_size = nerv_sizeof(_return_data_);</xsl:if>
            return_promise_t return_object = async_call(<xsl:choose><xsl:when test="in_data!='(null)'">_cal_data_,in_size,</xsl:when><xsl:when test="in_data='(null)'">0, 0,</xsl:when></xsl:choose> _limit_time);
            return async_return( return_object, <xsl:choose><xsl:when test="out_data!='(null)'">_return_data_,out_size,</xsl:when><xsl:when test="out_data='(null)'">0, 0,</xsl:when></xsl:choose> _time_ );
        }

        void <xsl:value-of select="name"/>_cancel(nerv_err_t _error)
        {
            cancel(_error);
        }
    };

    /* stub */
    class method_<xsl:value-of select="name"/> : public ::Nerv35::Stub_call
    {
    public:

        method_<xsl:value-of select="name"/>() : Stub_call(<xsl:value-of select="code"/>, <xsl:if test="out_data!='(null)'">sizeof(<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>)</xsl:if><xsl:if test="out_data='(null)'">0</xsl:if> ) {}

    protected:
        virtual ::Nerv35::nerv_err_t on_call(addr_t  _proxy_addr,
                                             const void*  _in_buf_,
                                             pksize_t     _in_buf_size,
                                             void*        _out_buf_,
                                             pksize_t*    _out_buf_size_,
                                             double       _elapsed_time,
                                             double       _packet_limit_time )
        {
            ::Nerv35::nerv_err_t r = on_<xsl:value-of select="name"/>(_proxy_addr,<xsl:if test="in_data!='(null)'"> (Nerv35::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>*)_in_buf_,</xsl:if><xsl:if test="out_data!='(null)'"> (Nerv35::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*)_out_buf_,</xsl:if> _elapsed_time, _packet_limit_time);
  <xsl:if test="out_data!='(null)'">
            if( ! r )
            {
                *_out_buf_size_ = nerv_sizeof((Nerv35::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*)_out_buf_);
            }
  </xsl:if>
            return r;
        }

        virtual void on_call_cancel( addr_t _proxy_addr, nerv_err_t _error)
        {
            on_<xsl:value-of select="name"/>_cancel(_proxy_addr, _error);
        }

        virtual ::Nerv35::nerv_err_t on_<xsl:value-of select="name"/>( addr_t _proxy_addr, <xsl:if test="in_data!='(null)'">const Nerv35::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _in_buf_,</xsl:if> <xsl:if test="out_data!='(null)'">Nerv35::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _out_buf_,</xsl:if> double _elapsed_time, double _packet_limit_time) = 0;

        virtual void on_<xsl:value-of select="name"/>_cancel(addr_t _proxy_addr, nerv_err_t _error) = 0;
    };
</xsl:template>

<xsl:template name ="template_signal" >
  <xsl:variable name="in_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="out_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>

    /* proxy */
    class proxy_method_<xsl:value-of select="name"/> : public ::Nerv35::Proxy_signal
    {
    public :
        proxy_method_<xsl:value-of select="name"/>() : Proxy_signal(<xsl:value-of select="code"/>) {}

        ::Nerv35::nerv_err_t <xsl:value-of select="name"/>(<xsl:if test="in_data != ''">const ::Nerv35::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _sig_data_,</xsl:if> double _limit_time = 1.0, double* _time_= 0)
        {
            <xsl:if test="in_data != ''">::Nerv35::pksize_t sig_size = nerv_sizeof(_sig_data_);</xsl:if>
            return signal(<xsl:choose><xsl:when test="in_data != ''">_sig_data_, sig_size,</xsl:when><xsl:when test="in_data = ''">0, 0,</xsl:when></xsl:choose> _limit_time, _time_);
        }

        void <xsl:value-of select="name"/>_cancel( nerv_err_t _error)
        {
            cancel(_error);
        }
    };

    /* stub */
    class method_<xsl:value-of select="name"/> : public ::Nerv35::Stub_signal
    {
    public:
        method_<xsl:value-of select="name"/>() : Stub_signal(<xsl:value-of select="code"/>) {}

    protected:
        virtual void on_signal( addr_t _proxy_addr,
                                const void* _buf_,
	                            pksize_t    _buf_size,
	                            double      _elapsed_time )
        {
            on_<xsl:value-of select="name"/>(_proxy_addr, <xsl:if test="in_data != ''">(<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>*)_buf_,</xsl:if> _elapsed_time);
        }

        virtual void on_signal_cancel( addr_t _proxy_addr, nerv_err_t _error)
        {
            on_<xsl:value-of select="name"/>_cancel(_proxy_addr,_error);
        }

        virtual void on_<xsl:value-of select="name"/>( addr_t _proxy_addr, <xsl:if test="in_data != ''">const ::Nerv35::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _buf_,</xsl:if> double _elapsed_time) = 0;

        virtual void on_<xsl:value-of select="name"/>_cancel( addr_t _proxy_addr, nerv_err_t _error) = 0;
    };
</xsl:template>

<xsl:template name ="template_event" >
  <xsl:variable name="in_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="out_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
    /* proxy */
    class proxy_method_<xsl:value-of select="name"/> : public ::Nerv35::Proxy_event
    {
    public :
        proxy_method_<xsl:value-of select="name"/>() : Proxy_event(<xsl:value-of select="code"/>) {}

        bool <xsl:value-of select="name"/>_is_connected() { return is_connected(); }

        ::Nerv35::nerv_err_t <xsl:value-of select="name"/>_link( double _limit_time = 1.0, double* _time_= 0 )
        {
            return_promise_t return_object = async_link( _limit_time );
            return async_link_return( return_object, _time_ );
        }

        void <xsl:value-of select="name"/>_unlink() { unlink(); }

        virtual void on_event( const void* _evt_data_, pksize_t _evt_size, double _elapsed_time)
        {
            on_<xsl:value-of select="name"/>(<xsl:if test="out_data!='(null)'">(Nerv35::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*)_evt_data_,</xsl:if> _elapsed_time);
        }

    private:
        virtual void on_unlink(nerv_err_t _err) { on_<xsl:value-of select="name"/>_unlink(_err); }

        virtual void on_<xsl:value-of select="name"/>( <xsl:if test="out_data!='(null)'">const Nerv35::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _evt_data_,</xsl:if> double _elapsed_time) = 0; //{}

        virtual void on_<xsl:value-of select="name"/>_unlink(nerv_err_t _err) = 0; //{}
    };

    /* stub */
    class method_<xsl:value-of select="name"/> : public ::Nerv35::Stub_event
    {
    public:
        method_<xsl:value-of select="name"/>() : Stub_event(<xsl:value-of select="code"/>) {}

        void <xsl:value-of select="name"/>_unlink(addr_t _proxy_addr ) { unlink(_proxy_addr); }

        void <xsl:value-of select="name"/>_unlink_all_proxy()
        {
            unlink_all_proxy();
        }

  <xsl:if test="out_data!='(null)'">
        nerv_err_t <xsl:value-of select="name"/>(Nerv35::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _buffer_)
        {
            ::Nerv35::pksize_t buf_size = nerv_sizeof(_buffer_);
            return event(_buffer_,buf_size);
        }
  </xsl:if>
  <xsl:if test="out_data='(null)'">
        nerv_err_t <xsl:value-of select="name"/>()
        {
            return event(0,0);
        }
  </xsl:if>

        virtual ::Nerv35::nerv_err_t on_link( addr_t _proxy_addr,
                                              double _packet_live_time,
                                              double _packet_limit_time )
        {
            return on_<xsl:value-of select="name"/>_link(_proxy_addr, _packet_live_time, _packet_limit_time);
        }

        virtual void on_unlink( addr_t _proxy_addr, nerv_err_t _err) { on_<xsl:value-of select="name"/>_unlink( _proxy_addr,_err); }

        virtual nerv_err_t on_<xsl:value-of select="name"/>_link( addr_t _proxy_addr,
                                                       double _packet_live_time,
                                                      double _packet_limit_time ) = 0;

        virtual void on_<xsl:value-of select="name"/>_unlink(addr_t _proxy_addr, nerv_err_t _err) = 0;
    };
</xsl:template>

<xsl:template name ="template_datagram" >
  <xsl:variable name="in_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
  <xsl:variable name="out_data_domain_namespace">
    <xsl:call-template name="get_data_domain_namespace">
      <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
    </xsl:call-template>
  </xsl:variable>
    /* proxy */
    class proxy_method_<xsl:value-of select="name"/> : public ::Nerv35::Proxy_datagram
    {
    public :
        proxy_method_<xsl:value-of select="name"/>() : Proxy_datagram(<xsl:value-of select="code"/>)
        {
        }

        virtual void on_<xsl:value-of select="name"/>_datagram( addr_t _stub_addr, const Nerv35::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _buf_, double _elapsed_time) = 0;

        virtual void on_datagram( addr_t _stub_addr, const void* _buf_, pksize_t _size, double  _elapsed_time)
        {
            on_<xsl:value-of select="name"/>_datagram(_stub_addr,(const Nerv35::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>*)_buf_, _elapsed_time);
        }
    };

    /* stub */
    class method_<xsl:value-of select="name"/> : public ::Nerv35::Stub_datagram
    {
    public:
        method_<xsl:value-of select="name"/>() : Stub_datagram(<xsl:value-of select="code"/>)
        {
        }

        nerv_err_t <xsl:value-of select="name"/>_broadcast( const ::Nerv35::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _data_)
        {
            ::Nerv35::pksize_t size = nerv_sizeof(_data_);
            return broadcast(_data_,size);
        }
    };
</xsl:template>




</xsl:stylesheet>
