<?xml version="1.0" encoding="utf-8"?>
<!-- copyright@ 2020, Waregram , All right Reserved. -->
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:include href="../../xsl_util.xsl"/>

<xsl:output method="text" encoding="utf-8" indent="yes"/>
<xsl:template match="/">
#include &lt;boost/python.hpp&gt;
#include &lt;Nerv35/nerv_utility.hh&gt;

// data export
<xsl:for-each select="/nerv_icde/datadomain/row">
#include &lt;Nerv35/Data<xsl:value-of select="path"/>/<xsl:value-of select="name"/>_export.h&gt;
</xsl:for-each>

// proxy export
<xsl:for-each select="/nerv_icde/component/proxy_list/row">
#include "Nerv/Proxy/<xsl:value-of select="class_name"/>/<xsl:value-of select="class_name"/>_export.h"
</xsl:for-each>

// stub export
<xsl:for-each select="/nerv_icde/component/stub_list/row">
#include "Nerv/Stub/<xsl:value-of select="class_name"/>/<xsl:value-of select="class_name"/>_export.h"
</xsl:for-each>

namespace py = boost::python;

void nerv_init()
{
    ::Nerv35::initialize(1, 1, 1);
}

void nerv_finalize()
{
    ::Nerv35::finalize();
}

::Nerv35::ip_t nerv_inet_addr(const char* str_ip)
{
    return ::Nerv35::inet_addr(str_ip);
}

// 도메인의 c_standard_port 얻는 함수로 제공
<xsl:for-each select="/nerv_icde/methoddomain/row">
int get_<xsl:value-of select="name"/>_port()
{
    <xsl:variable name="method_domain_namespace">
        <xsl:call-template name="get_method_domain_namespace">
            <xsl:with-param name="method_domain_id" select="id" />
        </xsl:call-template>
    </xsl:variable>

    return Nerv35::Method<xsl:value-of select="$method_domain_namespace"/>::c_standard_port;
}
</xsl:for-each>


void export_py_nerv()
{
    py::def("initialize", &amp;nerv_init);
    py::def("finalize", &amp;nerv_finalize);
    // py::def("dispatch_msg", &amp;nerv_dispatch_msg);
    py::def("inet_addr", &amp;nerv_inet_addr);

    py::class_&lt;::Nerv35::addr_t&gt;("addr_t")
        .def_readwrite("ip", &amp;::Nerv35::addr_t::ip)
        .def_readwrite("port", &amp;::Nerv35::addr_t::port)
        ;

    // export c_standard_port
    <xsl:for-each select="/nerv_icde/methoddomain/row">
    py::def("get_<xsl:value-of select="name"/>_port", &amp;get_<xsl:value-of select="name"/>_port);
    </xsl:for-each>

    // export data
    <xsl:for-each select="/nerv_icde/datadomain/row">
        <xsl:variable name="data_domain_namespace">
            <xsl:call-template name="get_data_domain_namespace">
                <xsl:with-param name="data_domain_id" select="id" />
            </xsl:call-template>
        </xsl:variable>
    <xsl:value-of select="$data_domain_namespace"/>::export_python_data();
    </xsl:for-each>

    // export proxy
    <xsl:for-each select="/nerv_icde/component/proxy_list/row">
    export_proxy_<xsl:value-of select="class_name"/>();
    </xsl:for-each>

    // export stub
    <xsl:for-each select="/nerv_icde/component/stub_list/row">
    export_stub_<xsl:value-of select="class_name"/>();
    </xsl:for-each>
}

BOOST_PYTHON_MODULE(py_nerv35)
{
    export_py_nerv();
}

<!-- int main()
{
    export_py_nerv();
} -->

</xsl:template>


</xsl:stylesheet>
