// 이 파일은 Nerv의

#include "nerv_define.h"

#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>

#if defined __cplusplus
extern "C" {
#endif

typedef int32_t  nerv_err_t;
typedef uint16_t nerv_port_t;
typedef uint32_t nerv_ip_t;
typedef int      nerv_inst_t;

typedef struct {
	nerv_ip_t   ip;
	nerv_port_t port;
} nerv_addr_t;

typedef uint16_t nerv_msg_code_t;
typedef uint32_t nerv_pksize_t;

typedef uint8_t         nerv_sync_model_t;
const nerv_sync_model_t nerv_on_post = 0x00U;
const nerv_sync_model_t nerv_on_timer = 0x01U;

// Stub delegate
typedef void on_nerv_stub_connect_ft(nerv_inst_t* _object_,
                                     nerv_addr_t  _proxy_addr);
typedef void on_nerv_stub_disconnect_ft(nerv_inst_t* _object_,
                                        nerv_addr_t  _proxy_addr);

// Stub_call delegate
typedef nerv_err_t on_nerv_stub_call_ft(nerv_inst_t*   _call_,
                                        nerv_addr_t    _proxy_addr,
                                        const void*    _in_buf_,
                                        nerv_pksize_t  _in_buf_size,
                                        void*          _out_buf_,
                                        nerv_pksize_t* _out_buf_size_,
                                        double         _elapsed_time,
                                        double         _packet_limit_time);

typedef void on_nerv_stub_call_cancel_ft(nerv_inst_t* _call_,
                                         nerv_addr_t  _proxy_addr,
                                         nerv_err_t   _error);

// Stub_command delegate
typedef void on_nerv_stub_command_sync_ft(nerv_inst_t*      _command_,
                                          nerv_addr_t       _proxy_addr,
                                          nerv_sync_model_t _sync_model,
                                          double            _elapsed_time,
                                          double            _limit,
                                          const void*       _buffer_,
                                          nerv_pksize_t     _size);

typedef nerv_err_t on_nerv_stub_command_link_ft(nerv_inst_t* _command_,
                                                nerv_addr_t  _proxy_addr,
                                                double       _packet_live_time,
                                                double _packet_limit_time);

typedef void on_nerv_stub_command_unlink_ft(nerv_inst_t* _object_,
                                            nerv_addr_t  _proxy_addr,
                                            nerv_err_t   _err);

// Stub_event delegate
typedef nerv_err_t on_nerv_stub_event_link_ft(nerv_inst_t* _event_,
                                              nerv_addr_t  _proxy_addr,
                                              double       _packet_live_time,
                                              double       _packet_limit_time);

typedef void on_nerv_stub_event_unlink_ft(nerv_inst_t* _event_,
                                          nerv_addr_t  _proxy_addr,
                                          nerv_err_t   _err);

// Stub_information delegate
typedef nerv_err_t on_nerv_stub_information_link_ft(nerv_inst_t* _information_,
                                                    nerv_addr_t  _proxy_addr,
                                                    bool         _reliability,
                                                    double _packet_live_time,
                                                    double _packet_limit_time);

typedef void on_nerv_stub_information_unlink_ft(nerv_inst_t* _information_,
                                                nerv_addr_t  _proxy_addr,
                                                nerv_err_t   _err);

// Stub_signal delegate
typedef void on_nerv_stub_signal_ft(nerv_inst_t*  _signal_,
                                          nerv_addr_t   _proxy_addr,
                                          const void*   _in_buf_,
                                          nerv_pksize_t _in_buf_size,
                                          double        _elapsed_time);

typedef void on_nerv_stub_signal_cancel_ft(nerv_inst_t* _signal_,
                                           nerv_addr_t  _proxy_addr,
                                           nerv_err_t   _error);

// Proxy delegate
typedef void on_nerv_proxy_connect_ft(nerv_inst_t* _object_);
typedef void on_nerv_proxy_disconnect_ft(nerv_inst_t* _object_);

// Proxy_command delegate
typedef void on_nerv_proxy_command_unlink_ft(nerv_inst_t* _command_,
                                             nerv_err_t   _err);

// Proxy_datagram delegate
typedef void on_nerv_proxy_datagram_cast_ft(nerv_inst_t*  _datagram_,
                                            nerv_addr_t   _stub_addr,
                                            const void*   _buf_,
                                            nerv_pksize_t _size,
                                            double        _elapsed_time);

// Proxy_event delegate
typedef void on_nerv_proxy_event_unlink_ft(nerv_inst_t* _event_,
                                           nerv_err_t   _err);

typedef void on_nerv_proxy_event_ft(nerv_inst_t*  _event_,
                                    const void*   _event_data_,
                                    nerv_pksize_t _event_size,
                                    double        _elapsed_time);

// Proxy_information delegate
typedef void on_nerv_proxy_information_unlink_ft(nerv_inst_t* _information_,
                                                 nerv_err_t   _err);

typedef void on_nerv_proxy_information_sync_ft(nerv_inst_t*      _information_,
                                               nerv_sync_model_t _sync_model,
                                               double            _elapsed_time,
                                               double            _limit,
                                               const void*       _buffer_,
                                               nerv_pksize_t     _size);

#if defined __cplusplus
} // extern "C" {
#endif

// // nerv_initialize
// NERV_SDK_DLL
// void nerv_initialize();
// NERV_SDK_DLL
// void nerv_finalize();

// // Stub
// NERV_SDK_DLL
// nerv_inst_t* Stub_alloc(on_nerv_stub_connect_ft*    _on_connect_,
//                                on_nerv_stub_disconnect_ft*
//                                _on_disconnect_);
// NERV_SDK_DLL
// void Stub_free(nerv_inst_t* _object_);
// NERV_SDK_DLL
// nerv_err_t Stub_start(nerv_inst_t* _object_,
//                              const char*  _name_,
//                              nerv_port_t* _port_);
// NERV_SDK_DLL
// nerv_err_t Stub_stop(nerv_inst_t* _object_);
// NERV_SDK_DLL
// bool Stub_is_started(nerv_inst_t* _object_);

// // Stub_datagram
// NERV_SDK_DLL
// nerv_inst_t* nerv_stub_datagram_alloc(nerv_msg_code_t _code,
//                                          nerv_inst_t*    _object);
// NERV_SDK_DLL
// void nerv_stub_datagram_free(nerv_inst_t* _object_);
// NERV_SDK_DLL
// nerv_err_t Stub_datagram(nerv_inst_t*  _object_,
//                                  const void*   _data_,
//                                  nerv_pksize_t _size);
// NERV_SDK_DLL
// nerv_err_t Stub_cast(nerv_inst_t*  _object_,
//                             nerv_ip_t     _ip,
//                             const void*   _data_,
//                             nerv_pksize_t _size);

// // Stub_call
// NERV_SDK_DLL
// nerv_inst_t* nerv_stub_call_alloc(nerv_msg_code_t    _code,
//                                     nerv_pksize_t      _max_return_size,
//                                     nerv_inst_t*       _object,
//                                     on_call_ft*        _on_call_,
//                                     on_call_cancel_ft* _on_call_cancel_);
// NERV_SDK_DLL
// void nerv_stub_call_free(nerv_inst_t* _object_);

// // Stub_command
// NERV_SDK_DLL
// nerv_inst_t*
//     nerv_stub_command_alloc(nerv_msg_code_t       _code,
//                               nerv_inst_t*          _object,
//                               on_commond_sync_ft*   _on_commond_sync_,
//                               on_command_link_ft*   _on_command_link_,
//                               on_command_unlink_ft* _on_command_unlink_);
// NERV_SDK_DLL
// void nerv_stub_command_free(nerv_inst_t* _object_);

// // Stub_event