#pragma once

/*
    ERROR 공통

    // 양수
    0x00000000~0x7FFFFFFF까지 도메인 구현에서 정의한다.         (도메인 에러)
    // 음수
    0x80000000~0xBFFFFFFF까지 예약한다.                         (통신 에러)
    0xC0000000~0xFFFFFFFF까지 미들웨어의 구현에서 정의한다.     (구현 에러)


    에러리턴값이 사용되는 곳
    함수호출 트랜잭션의 리턴 메시지
    트랜잭션 제어 리턴 메시지(연결 응답 메시지)

    공통에러
    모든 에러리턴에서 공통적인 의미로 사용됨, TCP/UDP의 에러리턴에 사용되는것은
   합법 객체와 미들웨어, 미들웨어와 미들웨어 모든 상황에서 공통적으로 발생가능한
   에러 정의 메시지타입+메시지코드 마다 에러 코드의 의미 통일 구현 정의 에러
    미들웨어의 구현에 따라서 사용되어지는 에러, TCP/UDP의 에러리턴에
   사용되는것은 불법 객체와 미들웨어 사이에서 발생하는 에러를 정의
    메시지타입+메시지코드 마다 에러 코드의 의미다를수 있음, 단 공통으로
   사용하는것을 추천 도메인 정의 에러 각 도메인에서 정의하는 에러, TCP/UDP의
   에러리턴에 사용되어지는 것은 합법 객체와 객체 사이에서 발생하는 에러를 정의
    도메인에서 정의하는 에러로서 각 타입,코드 별로 이 구간에서 에러코드의 의미는
   다르다.
*/

#include "nerv_typedef.hh"

namespace Nerv35 {

typedef int32_t nerv_err_t;

constexpr nerv_err_t NERV_ERR_NONE_ = 0x00000000U; // 에러 없음
constexpr nerv_err_t NERV_ERR_ALREADY_USED_NAME_ = 0x80000001U;
constexpr nerv_err_t NERV_ERR_OBJECT_IS_NOT_EXIST_ = 0x80000002U;
constexpr nerv_err_t NERV_ERR_NOT_SUPPORT_CODE_ = 0x80000003U;
constexpr nerv_err_t NERV_ERR_NOT_IMPLEMENT_ = 0x80000004U; // 구현되지 않음.
constexpr nerv_err_t NERV_ERR_REMOTE_TIMEOUT_ = 0x80000005U;
constexpr nerv_err_t NERV_ERR_BUSY_ = 0x80000006U;
constexpr nerv_err_t NERV_ERR_SHUTDOWN_ = 0x80000007U;

constexpr nerv_err_t NERV_ERR_LOCAL_TIMEOUT_ = 0xC0000000U;
constexpr nerv_err_t NERV_ERR_NOT_CONNECTED_TCP_ = 0xC0000001U;
constexpr nerv_err_t NERV_ERR_BOUNDOUT_BUFFER_ = 0xC0000002U;
constexpr nerv_err_t NERV_ERR_BUSY_LOCALHOST_ = 0xC0000003U;
constexpr nerv_err_t NERV_ERR_TOO_BIG_SIZE_ = 0xC0000004U;
constexpr nerv_err_t NERV_ERR_IS_NOT_CREATED_ = 0xC0000005U;
constexpr nerv_err_t NERV_ERR_NULL_RETURN_OBJECT_ = 0xC0000006U;
constexpr nerv_err_t NERV_ERR_ALREADY_CONNECTED_ = 0xC0000007U;
constexpr nerv_err_t NERV_ERR_ALREADY_CREATED_ = 0xC0000008U;
constexpr nerv_err_t NERV_ERR_DISCONNECTED_TCP_ = 0xC0000009U;
constexpr nerv_err_t NERV_ERR_DURTY_MEMORY_ = 0xC000000AU;
constexpr nerv_err_t NERV_ERR_OVERLAPED_PACKET_ = 0xC000000BU;

constexpr nerv_err_t NERV_ERR_DONT_GET_ = 0xD0000000U;
constexpr nerv_err_t NERV_ERR_EMPTY_NERV_MEMORY_ = 0xD0000001U;
constexpr nerv_err_t NERV_ERR_EMPTY_PROCESS_MEMORY_ = 0xD0000002U;
constexpr nerv_err_t NERV_ERR_NOT_EXIST_SYNC_DATA_ = 0xD0000003U;
constexpr nerv_err_t NERV_ERR_EMPTY_HISTORY_ = 0xD0000004U;
constexpr nerv_err_t NERV_ERR_ALREADY_USED_ADDR_ = 0xD0000005U;
constexpr nerv_err_t NERV_ERR_NOT_INITIALIZE_ = 0xD0000006U;
constexpr nerv_err_t NERV_ERR_UNKNOWN_ = 0xFFFFFFFFU; // 알 수 없는 에러

inline const char* get_err_str(nerv_err_t _err)
{
#define NERV_IF_EQ_RET_NAMEOF_(ERR)                                            \
	if(_err == ERR) return #ERR;
    NERV_IF_EQ_RET_NAMEOF_(NERV_ERR_NONE_);
    NERV_IF_EQ_RET_NAMEOF_(NERV_ERR_ALREADY_USED_NAME_);
    NERV_IF_EQ_RET_NAMEOF_(NERV_ERR_OBJECT_IS_NOT_EXIST_);
    NERV_IF_EQ_RET_NAMEOF_(NERV_ERR_NOT_SUPPORT_CODE_);
    NERV_IF_EQ_RET_NAMEOF_(NERV_ERR_NOT_IMPLEMENT_);
    NERV_IF_EQ_RET_NAMEOF_(NERV_ERR_REMOTE_TIMEOUT_);
    NERV_IF_EQ_RET_NAMEOF_(NERV_ERR_BUSY_);
    NERV_IF_EQ_RET_NAMEOF_(NERV_ERR_SHUTDOWN_);
    NERV_IF_EQ_RET_NAMEOF_(NERV_ERR_LOCAL_TIMEOUT_);
    NERV_IF_EQ_RET_NAMEOF_(NERV_ERR_NOT_CONNECTED_TCP_);
    NERV_IF_EQ_RET_NAMEOF_(NERV_ERR_BOUNDOUT_BUFFER_);
    NERV_IF_EQ_RET_NAMEOF_(NERV_ERR_BUSY_LOCALHOST_);
    NERV_IF_EQ_RET_NAMEOF_(NERV_ERR_TOO_BIG_SIZE_);
    NERV_IF_EQ_RET_NAMEOF_(NERV_ERR_IS_NOT_CREATED_);
    NERV_IF_EQ_RET_NAMEOF_(NERV_ERR_NULL_RETURN_OBJECT_);
    NERV_IF_EQ_RET_NAMEOF_(NERV_ERR_ALREADY_CONNECTED_);
    NERV_IF_EQ_RET_NAMEOF_(NERV_ERR_ALREADY_CREATED_);
    NERV_IF_EQ_RET_NAMEOF_(NERV_ERR_DISCONNECTED_TCP_);
    NERV_IF_EQ_RET_NAMEOF_(NERV_ERR_DURTY_MEMORY_);
    NERV_IF_EQ_RET_NAMEOF_(NERV_ERR_OVERLAPED_PACKET_);
    NERV_IF_EQ_RET_NAMEOF_(NERV_ERR_DONT_GET_);
    NERV_IF_EQ_RET_NAMEOF_(NERV_ERR_EMPTY_NERV_MEMORY_);
    NERV_IF_EQ_RET_NAMEOF_(NERV_ERR_EMPTY_PROCESS_MEMORY_);
    NERV_IF_EQ_RET_NAMEOF_(NERV_ERR_NOT_EXIST_SYNC_DATA_);
    NERV_IF_EQ_RET_NAMEOF_(NERV_ERR_EMPTY_HISTORY_);
    NERV_IF_EQ_RET_NAMEOF_(NERV_ERR_ALREADY_USED_ADDR_);
    NERV_IF_EQ_RET_NAMEOF_(NERV_ERR_NOT_INITIALIZE_);
    NERV_IF_EQ_RET_NAMEOF_(NERV_ERR_UNKNOWN_);
#undef NERV_IF_EQ_RET_NAMEOF_

	return "NERV_ERR_UNKNOWN_";
}

} // namespace Nerv35
