#pragma once

#include "nerv_define.h"
#include "nerv_typedef.hh"

namespace Nerv35 {

template <class TYPE> struct template_history_elem {
	TYPE*        buffer_;
	pksize_t     size;
	double       interval;
	nerv_clock_t clock;
};

struct struct_letter_case;

typedef struct_letter_case* letter_case_t;

typedef class NERV_SDK_DLL nerv_history {
public:
	nerv_history();
	nerv_history(const nerv_history& destructure);
	nerv_history(letter_case_t _letter_case);
	~nerv_history();
	                            operator bool();
	nerv_history&               operator=(letter_case_t _letter_case);
	nerv_history&               operator=(nerv_history& destructure);
	template_history_elem<void> operator[](unsigned int _index);
	size_t                      size();
	void                        clear();
	bool                        empty();

private:
	letter_case_t letter_case;
} destructor_t;

template <class TYPE> class template_history : public nerv_history {
public:
	template_history() {}
	template_history(const template_history& destructure)
	    : nerv_history(destructure)
	{}
	template_history(letter_case_t _letter_case) : nerv_history(_letter_case) {}
	                  operator bool() { return *(nerv_history*)(this); }
	template_history& operator=(letter_case_t _letter_case)
	{
		*((nerv_history*)(this)) = _letter_case;
		return (*this);
	}
	template_history& operator=(template_history& destructure)
	{
		*(nerv_history*)(this) = *(nerv_history*)&destructure;
		return (*this);
	}
	template_history_elem<TYPE> operator[](unsigned int _index)
	{
		// template_history_elem<void> void_elem =
		// ((nerv_history)(*this))[_index];
		template_history_elem<void> void_elem =
		    (*((nerv_history*)(this)))[_index];
		template_history_elem<TYPE> elem;
		elem.buffer_ = (TYPE*)void_elem.buffer_;
		elem.size = void_elem.size;
		elem.clock = void_elem.clock;
		elem.interval = void_elem.interval;
		return elem;
	}
	size_t size() { return nerv_history::size(); }
	void   clear() { nerv_history::clear(); }
	bool   empty() { return nerv_history::empty(); }
};

} // namespace Nerv35
