#pragma once

namespace Nerv35 {
// real -> 8bit unsigned integer
template <const double MIN(), const double MAX()>
inline void realtoscaledinteger(double _real_val, uint8_t& __int_val)
{
	if(_real_val < MIN()) { __int_val = 0; }
	else if(_real_val > MAX()) {
		__int_val = 255;
	}
	else {
		__int_val = (int8_t)((_real_val - MIN()) * ((255) / (MAX() - MIN())));
	}
}

// 8bit unsigned integer -> real
template <const double MIN(), const double MAX()>
inline void scaledintegertoreal(uint8_t _int_val, double& __real_val)
{
	__real_val = _int_val * ((MAX() - MIN()) / 255) + MIN();
}

// real -> 8bit signed integer
template <const double MIN(), const double MAX()>
inline void realtoscaledinteger(double _real_val, int8_t& __int_val)
{
	if(_real_val < MIN()) { __int_val = -127; }
	else if(_real_val > MAX()) {
		__int_val = 127;
	}
	else {
		__int_val = (int8_t)((_real_val - (MAX() + MIN()) / 2.0) * 2.0 *
		                     (127 / (MAX() - MIN())));
	}
}

// 8bit signed integer -> real
template <const double MIN(), const double MAX()>
inline void scaledintegertoreal(int8_t _int_val, double& __real_val)
{
	__real_val =
	    _int_val / 2.0 * ((MAX() - MIN()) / 127) + (MAX() + MIN()) / 2.0;
}

// real -> 16bit unsigned integer
template <const double MIN(), const double MAX()>
inline void realtoscaledinteger(double _real_val, uint16_t& __int_val)
{
	if(_real_val < MIN()) { __int_val = 0; }
	else if(_real_val > MAX()) {
		__int_val = 65535;
	}
	else {
		__int_val =
		    (int16_t)((_real_val - MIN()) * ((65535) / (MAX() - MIN())));
	}
}

// 16bit unsigned integer -> real
template <const double MIN(), const double MAX()>
inline void scaledintegertoreal(uint16_t _int_val, double& __real_val)
{
	__real_val = _int_val * ((MAX() - MIN()) / 65535) + MIN();
}

// real -> 16bit signed integer
template <const double MIN(), const double MAX()>
inline void realtoscaledinteger(double _real_val, int16_t& __int_val)
{
	if(_real_val < MIN()) { __int_val = -32767; }
	else if(_real_val > MAX()) {
		__int_val = 32767;
	}
	else {
		__int_val = (int16_t)((_real_val - (MAX() + MIN()) / 2.0) * 2.0 *
		                      (32767 / (MAX() - MIN())));
	}
}

// 16bit signed integer -> real
template <const double MIN(), const double MAX()>
inline void scaledintegertoreal(int16_t _int_val, double& __real_val)
{
	__real_val =
	    _int_val / 2.0 * ((MAX() - MIN()) / 32767) + (MAX() + MIN()) / 2.0;
}

// real -> 32bit unsigned integer
template <const double MIN(), const double MAX()>
inline void realtoscaledinteger(double _real_val, uint32_t& __int_val)
{
	if(_real_val < MIN()) { __int_val = 0; }
	else if(_real_val > MAX()) {
		__int_val = 0xFFFFFFFF;
	}
	else {
		__int_val =
		    (int32_t)((_real_val - MIN()) * ((0xFFFFFFFF) / (MAX() - MIN())));
	}
}

// 32bit unsigned integer -> real
template <const double MIN(), const double MAX()>
inline void scaledintegertoreal(uint32_t _int_val, double& __real_val)
{
	__real_val = _int_val * ((MAX() - MIN()) / 0xFFFFFFFF) + MIN();
}

// real -> 32bit signed integer
template <const double MIN(), const double MAX()>
inline void realtoscaledinteger(double _real_val, int32_t& __int_val)
{
	if(_real_val < MIN()) { __int_val = -2147483647; }
	else if(_real_val > MAX()) {
		__int_val = 2147483647;
	}
	else {
		__int_val = (int32_t)((_real_val - (MAX() + MIN()) / 2.0) * 2.0 *
		                      (2147483647 / (MAX() - MIN())));
	}
}

// 32bit signed integer -> real
template <const double MIN(), const double MAX()>
inline void scaledintegertoreal(int32_t _int_val, double& __real_val)
{
	__real_val =
	    _int_val / 2.0 * ((MAX() - MIN()) / 2147483647) + (MAX() + MIN()) / 2.0;
}

// real -> 64bit unsigned integer
template <const double MIN(), const double MAX()>
inline void realtoscaledinteger(double _real_val, uint64_t& __int_val)
{
	if(_real_val < MIN()) { __int_val = 0; }
	else if(_real_val > MAX()) {
		__int_val = 0;
		__int_val = ~__int_val;
	}
	else {
		uint64_t max_int = 0;
		max_int = ~max_int; // max_int = 18446744073709551615
		__int_val =
		    (uint64_t)((_real_val - MIN()) * ((max_int) / (MAX() - MIN())));
	}
}

// 64bit unsigned integer -> real
template <const double MIN(), const double MAX()>
inline void scaledintegertoreal(uint64_t _int_val, double& __real_val)
{
	uint64_t max_int = 0;
	max_int = ~max_int; // max_int = 18446744073709551615
	__real_val = _int_val * ((MAX() - MIN()) / max_int) + MIN();
}

// real -> 64bit signed integer
template <const double MIN(), const double MAX()>
inline void realtoscaledinteger(double _real_val, int64_t& __int_val)
{
	if(_real_val < MIN()) {
		int64_t max_int = 1;
		max_int <<= 63;
		max_int++;
		// max_int = -9223372036854775807
		__int_val = max_int;
	}
	else if(_real_val > MAX()) {
		int64_t max_int = 0;
		max_int = (uint64_t)~max_int >> 1;
		// max_int = 9223372036854775807;   0x7FFFFFF... = (ffffffff...>>1);
		__int_val = max_int;
	}
	else {
		int64_t max_int = 0;
		max_int = (uint64_t)~max_int >> 1;
		__int_val = (int64_t)((_real_val - (MAX() + MIN()) / 2.0) * 2.0 *
		                      (max_int / (MAX() - MIN())));
	}
}

// 64bit signed integer -> real
template <const double MIN(), const double MAX()>
inline void scaledintegertoreal(int64_t _int_val, double& __real_val)
{
	int64_t max_int = 0;
	max_int = (uint64_t)~max_int >> 1;
	__real_val =
	    _int_val / 2.0 * ((MAX() - MIN()) / max_int) + (MAX() + MIN()) / 2.0;
}

inline void initialize_real_integer(uint8_t& __integer) { __integer = 0; }
inline void initialize_real_integer(uint16_t& __integer) { __integer = 0; }
inline void initialize_real_integer(uint32_t& __integer) { __integer = 0; }
inline void initialize_real_integer(uint64_t& __integer) { __integer = 0; }

inline void initialize_real_integer(int8_t& __integer)
{
	*(uint8_t*)&__integer = 0x80;
}
inline void initialize_real_integer(int16_t& __integer)
{
	*(uint16_t*)&__integer = 0x8000;
}
inline void initialize_real_integer(int32_t& __integer)
{
	*(uint32_t*)&__integer = 0x80000000;
}
inline void initialize_real_integer(int64_t& __integer)
{
	*(uint64_t*)&__integer = 0x80000000;
	__integer <<= 32U;
}

inline bool is_valid_real_integer(uint8_t& __integer) { return true; }
inline bool is_valid_real_integer(uint16_t& __integer) { return true; }
inline bool is_valid_real_integer(uint32_t& __integer) { return true; }
inline bool is_valid_real_integer(uint64_t& __integer) { return true; }

inline bool is_valid_real_integer(int8_t& __integer)
{
	return *(uint8_t*)&__integer != 0x80;
}
inline bool is_valid_real_integer(int16_t& __integer)
{
	return *(uint16_t*)&__integer != 0x8000;
}
inline bool is_valid_real_integer(int32_t& __integer)
{
	return *(uint32_t*)&__integer != 0x80000000;
}
inline bool is_valid_real_integer(int64_t& __integer)
{
	int64_t comp_integer = 0;
	*(uint64_t*)&comp_integer = 0x80000000U;
	*(uint64_t*)&comp_integer <<= 32U;
	return __integer != comp_integer;
}

// data type scaled integer for network transmission.
// if pack size is not 1 , generated packed problem on arm g++.
// So that, this template class is that pack size is 1.
// generated warning when packed struct has member by normal struct.
// Really , this warning become to bug on arm Linux.
// so, our inserted packed syntax at real template.
// additionally, this inserting is generating compile time error.
//#pragma pack(1)
template <class TYPE, const double MIN(), const double MAX()>
class __attribute__((packed)) real {
public:
	TYPE int_val;

public:
	// scaled integer가 packed struct의 배열에 들어갈때는

	// real()
	//{
	//    initialize_real_integer(int_val);
	//}
	// real(double _real_val) {
	//    *this = _real_val;
	//}

	void        set_invalid() { initialize_real_integer(int_val); }
	bool        is_valid() { return is_valid_real_integer(int_val); }
	const real& operator=(double _real_val)
	{
		// 어느 플랫폼의 g++이든 Packed 문제가 발생한다.
		// packed의 멤버를 참조전달로 넘기는것에 대하여 컴파일 에러 처리함
		// 따라서 지역 변수를 참조변수로 넘기고 그 결과를 packed의 멤버
		// integer_value에 저장하는 형식을 취한다. 다른 컴파일러에서는 항상
		// 오버헤드로 처리하겠지만 미미하므로 컴파일러간 문장 분리 없이 코드
		// 적용

		// realtoscaledinteger<MIN,MAX>(_real_val,int_val); 이 구문은
		// 컴파일에러

		TYPE iv;
		realtoscaledinteger<MIN, MAX>(_real_val, iv);
		int_val = iv;

		return *this;
	}
	operator const double() const
	{
		// 다음 구문에서는 packed의 멤버가 값으로 전달되므로 위의 문제가
		// 발생하지 않는다. 당연히 그대로 둠
		double _real_val;
		scaledintegertoreal<MIN, MAX>(int_val, _real_val);
		return _real_val;
	}
	operator const int() const
	{
		// 다음 구문에서는 packed의 멤버가 값으로 전달되므로 위의 문제가
		// 발생하지 않는다. 당연히 그대로 둠
		double _real_val;
		scaledintegertoreal<MIN, MAX>(int_val, _real_val);
		return (int)(_real_val > 0 ? (_real_val + 0.5) : (_real_val - 0.5));
	}
	double get_value() const { return *this; }
};
//#pragma pack()

} // namespace Nerv35
